<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title: 3D Stars </H3>
            <p>Author: Gareth Noyce (g at korruptor.demon.co.uk)<br>
            Submission date: January 19, 2002</p>
	  
            <p>Description: A 3-D starfield effect, with the stars growing brighter as they approach.</p>
            
            <p align="center"><img src="3d-stars.png" width="320" height="200"></p>

            <p>Download: <a href="3d-stars.zip">3d-stars.zip</a></p>	  

            <p>pygame version required: Any (with surfarray)<br>
            SDL version required: Any<br>
            Python version required: 2.0 or newer (with Numeric)</p>

            <p>Comments: This effect is very similar to the one provided in the stars.py script in the pygame examples directory, with one difference - The stars vary in brightness depending on their distance.  Mr. Noyce explains his conceptual approach to the problem in the 3d-stars.readme file in the .zip archive.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">""" 3d-stars.py -- Korruptor Jan 2001, test code for Seal Basher (v1.0)

This is an example of a 3D star field, another staple demo effect... I don't think
it's very elagent, neat or optimised, but it shows how 3D starfields are made.

Interesting things to do: 
    add rotation, 
    do it for all axis,
    do progressive erasure so the 'closer' the star is to the screen the larger the trail behind it,
    make close stars fater,
    do it with sprites instead of pixel fills...
    etc."""</span>
    
import pygame, pygame.transform
from random import *
from pygame.surfarray import *
from pygame.locals import *
from Numeric import *

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pycomment"># Glob decs</span>

<span class="pycomment"># Screen resolution...</span>
RES 	= array((320,200))

<span class="pycomment"># We use this to provide the upper and lower bands of the initial x,y randomisation</span>
<span class="pycomment"># and then as a modifier to shift negative x,y values back into our screen display range </span>
RANGE_X   = RES[0]/2
RANGE_Y   = RES[1]/2

<span class="pycomment"># An array of star positions Z,X,Y -- [In|De]crease length for different star numbers...</span>
STARS      = zeros((300,3))
COORDS     = zeros((300,2))

<span class="pycomment"># Initialise an array for storing RGB tuples</span>
COLOUR_MAP = [[]] * 256

<span class="pycomment"># Constants...</span>
global MAXSTARS
global MAXZ  
global NUMSTARS 

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def main():</span>
    <span class="pycomment">"Inisalises display, precalculates the cosine values, and controls the update loop"</span>
    
    <span class="pycomment"># Change this and the length of the STARS array for more stars...</span>
    MAXSTARS = 300
    <span class="pycomment"># The number of live stars...</span>
    NUMSTARS = 0 
    
    <span class="pycomment"># Initialise pygame, and grab an 8bit display.</span>
    pygame.init()
    screen_surface = pygame.display.set_mode(RES, 0, 8)

    <span class="pycomment"># setup the screen palette...</span>
    for i in range(0,256):
        COLOUR_MAP[i] = (256-i),(256-i),(256-i)

    <span class="pycomment"># Slap the palette onto our display</span>
    screen_surface.set_palette(COLOUR_MAP)


    <span class="pycomment"># Create an initial star set...</span>
    NUMSTARS = create_stars(NUMSTARS,MAXSTARS)
    
    <span class="pycomment"># Fruity loops...</span>
    while 1:
    
        <span class="pycomment"># Have we received an event to close the window?</span>
        for e in pygame.event.get():
            if e.type in (QUIT,KEYDOWN,MOUSEBUTTONDOWN):
                return

        <span class="pycomment"># Right, check for dead stars and make some new ones...</span>
        NUMSTARS = create_stars(NUMSTARS, MAXSTARS)
        <span class="pycomment"># Then update our star positions...</span>
        NUMSTARS = update_stars(NUMSTARS, MAXSTARS,screen_surface)
        <span class="pycomment"># Show the results to our audience...</span>
        pygame.display.update()

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def update_stars(nstars,mstars,screen):</span>
    <span class="pycomment">"Erase old stars, and update the new ones before drawing them again..."</span>

    <span class="pycomment"># Loop through each star...</span>
    for i in range(0,mstars):

        <span class="pycomment"># Check the z-value for the star. If it's 0 we know we've found a dead star...</span>
        if(STARS[i][0] == 0):
            <span class="pycomment"># Ignore this and check the next star...</span>
            nstars - 1
            continue
        else:
            <span class="pycomment"># Ooo, we gotta live one. Erase it...</span>
            screen.fill((0,0,0), (COORDS[i][0], COORDS[i][1], 1, 1))            
        
            <span class="pycomment"># Calculate the new x/y coords for the star (This bitta maths nicked from Vulture/OUTLAW)...</span>
            COORDS[i][0] = ((256 * STARS[i][1]) / STARS[i][0]) + RANGE_X
            COORDS[i][1] = ((256 * STARS[i][2]) / STARS[i][0]) + RANGE_Y
                    
            <span class="pycomment"># Bounds check the values for x and y... Kill the star if it's offscreen...</span>
            if((COORDS[i][0] > (RES[0])) or (COORDS[i][1] > (RES[1]))):
                STARS[i][0] = 0
                STARS[i][1] = 0
                STARS[i][2] = 0
                <span class="pycomment"># Decrement the number of live stars so the next call to create_stars will make a new one...</span>
                nstars -= 1
                continue
        
            <span class="pycomment"># Draw the star's new position to the screen using the colour specified in the colour_map...</span>
            screen.fill(COLOUR_MAP[STARS[i][0]], (COORDS[i][0], COORDS[i][1], 1, 1))            

            <span class="pycomment"># Decrement the z value for the star... (Increment to zoom out)</span>
            <span class="pycomment"># Not checking for the z value == 1 caused a bastard bud I've been hunting all night :)</span>
            if (STARS[i][0] < 4):
                STARS[i][0] -= 4
                nstars -= 1
            else:
                STARS[i][0] -= 4
                
    return nstars
<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def create_stars(nstars, mstars):</span>
    <span class="pycomment">"Check for empty array slots and create a new star in there..."</span>

    <span class="pycomment"># If we've got enough stars already...</span>
    if(nstars >= mstars):
        <span class="pycomment"># Don't do anything...</span>
        return nstars
    else:
        <span class="pycomment"># Find the dead star...</span>
        for i in range(0,mstars):
            if(STARS[i][0] < 1):
                <span class="pycomment"># Create a new star in it's place...</span>
                STARS[i][0] = 255
                STARS[i][1] = randrange(negative(RANGE_X),RANGE_X,1)
                STARS[i][2] = randrange(negative(RANGE_Y),RANGE_Y,1)
                <span class="pycomment"># Increment that number of live stars...</span>
                nstars += 1
    return nstars
<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pycomment"># ------------------------------------------------------------------------------------</span>

<span class="pycomment"># Ahead warp fact 1 Mr Sulu...</span>
if __name__ == '<span class="pystring">__main__</span>': main()

<span class="pycomment"># End of sauce. Pass the chips...</span>

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

