<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  3-D Wireframe example</H3>
            <p>Author: Gustavo Niemeyer (niemeyer at conectiva.com)<br>
            Submission date: December 2, 2001</p>
	  
            <p>Description: 3-D Wireframe graphics example. Mr. Niemeyer uses a draw_3dline function to render a 3-d wireframe object without 3-d hardware or opengl. The result is reminiscent of the graphics of the earliest Elite games.  A cube is rendered, then spun about its origin.</p>
            
            <p align="center"><img src="2dcube.gif" width="320" height="200"></p>

            <p>Download: <a href="2dcube.tar.gz">2dcube.tar.gz</a>, <a href="2dcube.zip">2dcube.zip</a></p>	  

            <p>pygame version required: 0.9 or later<br>
            SDL version required: Any<br>
            Python version required: 2.0 or later</p>

            <p>Comments: To define a given solid, you'll have to specify your edges.  This script does so by creating a list of verticies, where each vertex is a xyz coordinate.  It then calls a function which defines edges between certain vertecies, and draws them.  The rotate object function is more general, and takes transformation parameters similar to those used by OpenGL.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">#!/usr/bin/python
"""
This example shows a simple wireframe rotating cube.

Written by Gustavo Niemeyer &lt;niemeyer@conectiva.com&gt;.
"""</span>
from pygame.locals import *
import pygame.draw
import pygame.time
from math import sin, cos

ORIGINX = 0
ORIGINY = 0

<span class="pydef">def draw_3dline(surface, color, a, b):</span>
	<span class="pycomment">"""Convert 3D coordinates to 2D and draw line."""</span> 
	ax, ay = a[0]+(a[2]*0.3)+ORIGINX, a[1]+(a[2]*0.3)+ORIGINY
	bx, by = b[0]+(b[2]*0.3)+ORIGINX, b[1]+(b[2]*0.3)+ORIGINY
	pygame.draw.line(surface, color, (ax, ay), (bx, by))

<span class="pydef">def draw_cube(surface, color, cube):</span>
	<span class="pycomment">"""Draw 3D cube."""</span>
	a, b, c, d, e, f, g, h = cube
	draw_3dline(surface, color, a, b)
	draw_3dline(surface, color, b, c)
	draw_3dline(surface, color, c, d)
	draw_3dline(surface, color, d, a)
	
	draw_3dline(surface, color, e, f)
	draw_3dline(surface, color, f, g)
	draw_3dline(surface, color, g, h)
	draw_3dline(surface, color, h, e)
	
	draw_3dline(surface, color, a, e)
	draw_3dline(surface, color, b, f)
	draw_3dline(surface, color, c, g)
	draw_3dline(surface, color, d, h)

<span class="pydef">def rotate_3dpoint(p, angle, axis):</span>
	<span class="pycomment">"""Rotate a 3D point around given axis."""</span>
	ret = [0, 0, 0]
	cosang = cos(angle)
	sinang = sin(angle)
	ret[0] += (cosang+(1-cosang)*axis[0]*axis[0])*p[0]
	ret[0] += ((1-cosang)*axis[0]*axis[1]-axis[2]*sinang)*p[1]
	ret[0] += ((1-cosang)*axis[0]*axis[2]+axis[1]*sinang)*p[2]
	ret[1] += ((1-cosang)*axis[0]*axis[1]+axis[2]*sinang)*p[0]
	ret[1] += (cosang+(1-cosang)*axis[1]*axis[1])*p[1]
	ret[1] += ((1-cosang)*axis[1]*axis[2]-axis[0]*sinang)*p[2]
	ret[2] += ((1-cosang)*axis[0]*axis[2]-axis[1]*sinang)*p[0]
	ret[2] += ((1-cosang)*axis[1]*axis[2]+axis[0]*sinang)*p[1]
	ret[2] += (cosang+(1-cosang)*axis[2]*axis[2])*p[2]
	return ret

<span class="pydef">def rotate_object(obj, angle, axis):</span>
	<span class="pycomment">"""Rotate an object around given axis."""</span>
	for i in range(len(obj)):
		obj[i] = rotate_3dpoint(obj[i], angle, axis)

<span class="pydef">def main():</span>
	global ORIGINX, ORIGINY
	pygame.init()
	screen = pygame.display.set_mode((320,200))
	<span class="pycomment"># Move origin to center of screen</span>
	ORIGINX = screen.get_width()/2
	ORIGINY = screen.get_height()/2
	cube = [(-50,50,50),  (50,50,50),  (50,-50,50),  (-50,-50,50),
			(-50,50,-50), (50,50,-50), (50,-50,-50), (-50,-50,-50)]
	while 1:
		draw_cube(screen, 255, cube)
		event = pygame.event.poll()
		if event.type == QUIT or (event.type == KEYDOWN and event.key == K_ESCAPE):
			break
		pygame.display.flip()
		pygame.time.delay(25)
		draw_cube(screen, 0, cube)
		rotate_object(cube, 0.1, (0,1,0))
		rotate_object(cube, 0.01, (0,0,1))
		rotate_object(cube, 0.01, (1,0,0))

if __name__ == "__main__":
	main()


</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

