<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title: Bitmapped font module </H3>
            <p>Author: Paul Sidorsky  (paulsid at home.com)<br>
            Submission date: June 5, 2001</p>
	  
            <p>Description: A simple class for bitmapped fonts in pygame. A bitmapped font is simply a font specified as a bitmap. Bitmapped fonts are easy to use, fast, and let you display fancy multicoloured text.  The characters are all the same size, so it's easy to treat the screen as a matrix of character cells, as if you were in text mode (e.g. for a status screen between levels).  Drawbacks are that you must of course draw the font yourself, you can't change the colour, and you don't get special effects like bold, italics, etc.</p>
            
            <p align="center"><img src="bmpfont.gif" width="400" height="300"></p>

            <p>Download: <a href="bmpfont.zip">bmpfont.zip</a></p>	  

            <p>pygame version required: 1.0<br>
            SDL version required: Any<br>
            Python version required: 2.1</p>

            <p>Comments: This submission is so well documented, I don't have much to add. As Mr. Sidorsky says, this file package allows you to create your own bitmapped font image, define it with an index file, then blit it to your surface.  The included code uses xmodelines(), which is only available in Python 2.1; to make it work with Python 2, you must change line 77 from:</p>

<pre>for x in f.xreadlines():</pre>

<p>to:</p>

<pre>for x in f.readlines():</pre>

<p>The result will be a little slower, but will be compatible with older versions of python.</p>

            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment"># bmpfont.py
# By Paul Sidorsky - Freeware

"""Provides support for bitmapped fonts using pygame.

Font Index File Descrption

bmpfont lets you define where each character is within the bitmap,
along with some other options.	This lets you use a bitmap of any
dimension with characters of any size.	The file where the position
of each character is defined is called the font index file.  It is
a simple text file that may contain the lines listed below.
Whitespace is ignored, but the keywords are case-sensitive.
NOTE:  Blank lines and comments are not allowed!

bmpfile filename
- filename is the full name (with optional path) of the bitmap file
  to use.  The file can be of any type supported by pygame.image.
  Defaults to font.bmp with no path if omitted.

width x
height y
- Specifies the dimensions of a character, in pixels.  Each
  character must be of the same width and height.  width defaults
  to 8 and height to 16 if omitted.

transrgb r g b
- Specifies the colour being used to indicate transparency.  If you
  don't wish to use transparency, set this to an unused colour.
  Defaults to black (0, 0, 0) if omitted.

alluppercase
- If present, indicates the font only has one case of letters which
  are specified in the index using upper case letters.	Strings
  rendered with BmpFont.blit() will be converted automatically.  If
  omitted, the font is assumed to have both cases of letters.

- All other lines are treated as character index specifiers with
  the following format:

  char x y

  - char is the character whose position is being specified.  It
	can also be "space" (without quotes) to define a position for
	the space character.
  - x is the column number where char is located.  The position
	within the bitmap will be x * width (where width is specified
	above).
  - y is the row number where char is located.	The position will
	be y * height.
"""</span>

import pygame.image
from pygame.locals import *

__all__ = ["<span class="pystring">BmpFont</span>"]

<span class="pydef">class BmpFont:</span>
        <span class="pycomment">"""Provides an object for treating a bitmap as a font."""

	# Constructor - creates a BmpFont object.
	# Parameters:  idxfile - Name of the font index file.</span>
        <span class="pydef">def __init__(self, idxfile = "font.idx"):</span>
                <span class="pycomment"># Setup default values.</span>
                self.alluppercase = 0
                self.chartable = {}
                self.bmpfile = "<span class="pystring">font.bmp</span>"
                self.width = 8
                self.height = 16
                self.transrgb = (0, 0, 0)

                <span class="pycomment"># Read the font index.	File errors will bubble up to caller.</span>
                f = open(idxfile, "<span class="pystring">r</span>")

                for x in f.xreadlines():
                        <span class="pycomment"># Remove EOL, if any.</span>
                        if x[-1] == '<span class="pystring">\n</span>': x = x[:-1]
                        words = x.split()

                        <span class="pycomment"># Handle keywords.</span>
                        if words[0] == "<span class="pystring">bmpfile</span>":
                                self.bmpfile = x.split(None, 1)[1].strip()
                        elif words[0] == "<span class="pystring">alluppercase</span>":
                                self.alluppercase = 1
			elif words[0] == "<span class="pystring">width</span>":
				self.width = int(words[1])
			elif words[0] == "<span class="pystring">height</span>":
				self.height = int(words[1])
			elif words[0] == "<span class="pystring">transrgb</span>":
				self.transrgb = (int(words[1]), int(words[2]), int(words[3]))
			else:  <span class="pycomment"># Default to index entry.</span>
				if words[0] == "<span class="pystring">space</span>": words[0] = '<span class="pystring"> </span>'
				if self.alluppercase: words[0] = words[0].upper()
				self.chartable[words[0]] = (int(words[1]) * self.width, int(words[2]) * self.height)
		f.close()

                <span class="pycomment"># Setup the actual bitmap that holds the font graphics.</span>
                self.surface = pygame.image.load(self.bmpfile).convert()
                self.surface.set_colorkey(self.transrgb, RLEACCEL)

        <span class="pycomment"># blit() - Copies a string to a surface using the bitmap font.
        # Parameters:  string	 - The message to render.  All characters
        #						   must have font index entries or a
	#						   KeyError will occur.
	#			   surf 	 - The pygame surface to blit string to.
	#			   pos		 - (x, y) location specifying location
	#						   to copy to (within surf).  Meaning
	#						   depends on usetextxy parameter.
	#			   usetextxy - If true, pos refers to a character cell
	#						   location.  For example, the upper-left
	#						   character is (0, 0), the next is (0, 1),
	#						   etc.  This is useful for screens with
	#						   lots of text.  Cell size depends on the
	#						   font width and height.  If false, pos is
	#						   specified in pixels, allowing for precise
	#						   text positioning.</span>
        <span class="pydef">def blit(self, string, surf, pos = (0, 0), usetextxy = 1):</span>
                <span class="pycomment">"""Draw a string to a surface using the bitmapped font."""</span>
                x, y = pos
                if usetextxy:
                        x *= self.width
                        y *= self.height
                surfwidth, surfheight = surf.get_size()
                fontsurf = self.surface.convert(surf)

                if self.alluppercase: string = string.upper()

                <span class="pycomment"># Render the font.</span>
                for c in string:
                        <span class="pycomment"># Perform automatic wrapping if we run off the edge of the</span>
                        <span class="pycomment"># surface.</span>
                        if x >= surfwidth:
                                x -= surfwidth
                                y += self.height
                                if y >= surfheight:
                                        y -= surfheight

                        surf.blit(fontsurf, (x, y), (self.chartable[c], (self.width, self.height)))
                        x += self.width

<span class="pycomment"># Example code.  Run this file as a script to activate.</span>
if __name__ == "<span class="pystring">__main__</span>":
        import pygame
        pygame.init()
        screen = pygame.display.set_mode((400, 300), 0, 16)
        pygame.event.pump()
        <span class="pycomment"># Assumes font.idx and font.bmp are in the current directory.</span>
        bmpfont = BmpFont()
        msg = "<span class="pystring">ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz</span>"\
              "<span class="pystring">0123456789 !@#$%^&*()-=_+\|[]{};:'\",.<>/?`~</span>" * 5
	bmpfont.blit(msg, screen)
	pygame.display.flip()
	while pygame.event.poll().type != QUIT: pass
<span class="pycomment"># End of file bmpfont.py</span>

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

