<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Blinds Wipe Effect</H3>
            <p>Author: Gareth Noyce (g at korruptor.demon.co.uk)<br>
            Submission date: March 3, 2002</p>
	  
            <p>Description: A white-to-black 'blinds' transition effect</p>
            
            <p align="center"><img src="blinds.png" width="320" height="240"></p>

            <p>Download: <a href="blinds.zip">blinds.zip</a></p>	  

            <p>pygame version required: Any, with surfarray<br>
            SDL version required: Any<br>
            Python version required: Any, with Numeric</p>

            <p>Comments: Garath Noyce wows us with another Numeric-powered pygame effect.  This time it's a 'blinds' transition, familiar to anyone who's ever had to sit through a powerpoint presentation.  Here we fade from white to black, and from right to left - it should be pretty easy to do a bitmap fade, or to change the direction of the wipe.  The numeric code is as difficult to follow as usual, but Mr. Noyce's 'extreme' approach to commenting should guide you through.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""Blinds.py -- Gareth Noyce (aka Korruptor) v1.0 -- Testcode for Seal Basher...

   HTTP://www.korruptor.demon.co.uk

This is a little effect to produce a window blind effect. Another useful transtion...

This version just blits a white background and performs the blinds effect over the top
of that. It also uses an 8bit palette, however, it's easy to move to a real colou
version, and using numeric's subtract() function, you could easily draw the blinds
over a bitmap... Have a play...

The routine itself is fairly simple (as usual for me!):

We intially divvy up the screen into eight chunks (blind_cut).

We then draw an initial blind at the far right of the screen by drawing two black columns seperated by
blind_cut number of columns, we then fill these (fade) shades darker than what they where previously 

With this intial blind drawn, we can loop from left to right across the surfarray (jumping blind_cut number of cols), 
looking for black columns, which indicate the start of a blind. When we find one, we look for the blind inside the
black columns, and if found we decrease the blind's width by drawing two more black columns, one either side of the blind, 
and fill the center of the blind (fade) shades darker...

If we passed five black columns whilst looking for the inside of the blind, we draw a new blind to the left of the one 
we're working on, which will be picked up and worked on in the next iteration.

If we find no blind to fade, we skip the loop, unless we're at the left hand edge of the screen, in which case
we exit out of the function...

Phew! It's probably easier to read the code! ;-)


The function itself takes a surfarray, the dimensions of the array, and a surface to blit
the effect onto. It also doesn't return until it's complete, but it should be fairly
easy to remove this and control the function "more directly"...


>>> One thing to be aware of: Make sure the fade amount gives you enough 'steps' for the width of the blind on a given screen 
    resolution. Two large a number will see the blind pop back to a white fill half way through the transtion...


Have fun...


"""</span>


<span class="pycomment">#-------------------------------------------------------------------------------------</span>

import pygame
from pygame.surfarray import *
from pygame.locals import *
from Numeric import *


<span class="pycomment"># GLOBALS...</span>

RES = array((640,480))
COLOUR_MAP = [[]] * 256

<span class="pycomment">#-------------------------------------------------------------------------------------</span>

<span class="pydef">def main():</span>
	<span class="pycomment">" Initialises display, fills a dummy screen to white, calls the blind effect..."</span>


	<span class="pycomment"># Init pygame, grab an 8bit display...</span>
	pygame.init()

	<span class="pycomment">#setup a surface for the display, and a palette which we'll use...</span>
	screen_surface 	= pygame.display.set_mode(RES, 0, 8)

    	<span class="pycomment"># setup the screen palette... (Just greyscale 8bit for this hacK)	</span>
	for i in range(0,256):
        	COLOUR_MAP[i] = (i),(i),(i)


	<span class="pycomment"># Slap the palette onto the screen...</span>
	screen_surface.set_palette(COLOUR_MAP)

	<span class="pycomment"># Fill the screen with white, as we know we'll fade up to white for the effect, so this will always be our base colour...</span>
	screen_surface.fill((255,255,255))    

	<span class="pycomment"># Create a surfarray of the screen for hacking on...</span>
	screen_map = pygame.surfarray.array2d(screen_surface)

	<span class="pycomment"># Quick ping of the display...</span>
	pygame.display.update()

	<span class="pycomment"># Call the blinds function...</span>
	blinds(screen_map, RES, screen_surface, 6)
	
	<span class="pycomment"># Print out a little message, and exit...</span>
	print "<span class="pystring">Blinds transition, Korruptor 2002</span>"



<span class="pycomment">#-------------------------------------------------------------------------------------</span>
<span class="pydef">def blinds(map,size, screen, fade):</span>
	<span class="pycomment">"Using a 256 colour palette, this function will draw a blinds effect across the screen, leaving a black background..."</span>


	<span class="pycomment"># Calculate the width of each "blind"</span>
	blind_cut = size[0]/8

	<span class="pycomment">#:::::Initial setup for the first blind:::::</span>



	<span class="pycomment"># Draw a black column on the far right column of the screen</span>
	map[size[0]-1][0:size[1]] = 0

	<span class="pycomment"># Grab the colour on the column "inside" of what will become the blind </span>
	<span class="pycomment"># (white in this example, but I'm trying to be palette agnostic)...</span>
	current_col = map[size[0]-1][0]

	<span class="pycomment"># Fill the "inside" of the blind 'fade' shades darker...</span>
	for i in range (1,blind_cut):
		map[size[0]-(i+1)][0:size[1]]= current_col - fade

	<span class="pycomment"># Fill the start of this blind with black so we have something to find in our loop...</span>
	map[size[0]-blind_cut][0:size[1]]= 0



	<span class="pycomment">#:::::Initial Blind Setup Completed:::::</span>


	<span class="pycomment"># That's the first blind created, we now go into our "find and fill" loop until the screen has transitioned...</span>
	<span class="pycomment"># We'll trip this sentinal when wecan figure out we're finished...</span>
	fin = 0
	while not(fin):

		<span class="pycomment"># From left to right of the screen, move in blind_cut steps...</span>
		for i in range (0,size[0]-1,blind_cut):
			
			<span class="pycomment"># And see if we've found the start of a blind (a black column)</span>
			if (map[i][0]== 0):
				<span class="pycomment"># We've found the start of a blind!</span>
				j = 1

				<span class="pycomment"># move inward over the black columns...</span>
				<span class="pycomment"># until we find the blind which we'll need to fill...</span>
				while (j&lt;blind_cut) and (map[i+j][0]==0):
					j+=1
				


				<span class="pycomment">#-------------------------------------------------------------------------------------</span>
				<span class="pycomment"># Here's where we do our 'exit' checks, and prevent any out of bounds surfarray access</span>
				<span class="pycomment">#-------------------------------------------------------------------------------------</span>
					
				<span class="pycomment"># If we've traveresed the full width of a blind and found nothing to fill, we know </span>
				<span class="pycomment"># we can ignore this one and continue with the loop...</span>
				if (j == blind_cut):

					<span class="pycomment"># But! We need to make sure we're not at the far left of the screen...</span>
					<span class="pycomment"># if that's the case, we know we're done here... :-)</span>
					if(i==0):
						fin = 1
						break
					<span class="pycomment"># Ok, the transition isn't complete yet, so we'll just jump out of this loop...</span>
					else:
						break
				
				<span class="pycomment">#-------------------------------------------------------------------------------------
				# Here's the "business" end of the blinds routine...
				#-------------------------------------------------------------------------------------
				
				
				# we've found the inside of the blind, so, we decrease it's width
				# by adding a black coloumn to the left and right of the blind... </span>
				map[i+j][0:size[1]] = 0
				map[(i+blind_cut)-(j)][0:size[1]] = 0
	

				<span class="pycomment"># Now we've got to fill the inside of the blind, so grab it's current colour...</span>
				current_col = map[i+j+1][0]

				<span class="pycomment"># ...and fill the inside of the blind 'fade' shades darker...</span>
				for k in range((i+j+1),(i+(blind_cut-j))-1):
					map[k][0:size[1]] = current_col - fade

				<span class="pycomment"># if we've reached the criteria where we make a new blind</span>
				if j==5:
					<span class="pycomment"># start it off, this will give us something to trip over in the next iteration of the loop</span>
					map[i-blind_cut][0:size[1]]=0


				<span class="pycomment">#-------------------------------------------------------------------------------------</span>

		<span class="pycomment"># Blit this iteration to the screen and show it...</span>
		blit_array(screen, map)
		pygame.display.update()


#--------------------------------------------------------------------------------------

if __name__ == '<span class="pystring">__main__</span>': main()

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

