<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Caching Resource Module</H3>
            <p>Author: David Clark (da_clark at shaw.ca)<br>
            Submission date: January 14, 2002</p>
	  
            <p>Description: A module that ensures that only one copy of each graphic, font or sound resource is loaded into memory.</p>

            <p>Download: <a href="resource.py">resource.py</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: Any</p>

            <p>Comments:  Most game projects require graphics, sound and other resources to be loaded multiple times - a sprite's image might be loaded for each instance of that sprite.  Normally, you'll write some sort of resource management system to ensure that all these identical resources aren't kept in memory at once.  Resource.py is such a management system</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

import pygame, os
import pygame.image, pygame.font, pygame.mixer <span class="pycomment"># remove for pygame >=1.4</span>

<span class="pycomment">"""
resource.py
A caching resource loader
License: Public Domain
Author: David Clark (silenus at telus.net)

This module wraps pygame's resource loading functions.  It ensures that only
one copy of any given resource will be in memory at any time - further requests
for that same resource will get a reference to the loaded copy.  This behavoir
can be overridden by specifying that the resource loading be forced - all
clients of that resource will then refer to the new version.  The resource can
be manually removed from memory with the clear_* set of functions.

This code might be useful if a game is going to be loading many copies of a
given disk resource.  Most of the time, you'd write your own cache; now you
can just use this one.

Remember to call the 'set path' functions before trying to load anything;
otherwise, you'll get a ValueError.

"""</span>



__images = {}
__fonts = {}
__sounds = {}

__images_path = None
__fonts_path = None
__sounds_path = None

<span class="pydef">def get_image(filename, force_reload = 0):</span>
    <span class="pycomment">"""
    get_image(filename, force_reload = 1) --> surface
    Call this function instead of pygame.image.load - it will load the image
    from the disk the first time, then just return a reference to the copy each
    subsequent time.  This function does no colorkey setting or pixel format
    conversion; you'll have to do that manually, if you wish.
    """</span>
    
    if not __images_path:
        raise ValueError, "<span class="pystring">resources.set_images_path() not called yet.</span>"
    if (force_reload == 1 or filename not in __images.keys()):
        try:
            surface = pygame.image.load(os.path.join(__images_path, filename))
        except pygame.error:
            raise IOError, "<span class="pystring">File </span>" + filename + "<span class="pystring"> not found.</span>"
        __images[filename] = surface
        return surface
    else:
        return __images[filename]

<span class="pydef">def has_image(filename):</span>
    <span class="pycomment">"""
    has_image(filename) --> Boolean
    Returns true if the image is in memory, false if it has to be loaded from
    disk.
    """</span>
    return __images.has_key(filename)

<span class="pydef">def clear_image(filename):</span>
    <span class="pycomment">"""
    clear_image(filename) --> Boolean
    Eliminates the image from memory.  Subsequent calls will load it from the
    disk.  Returns True if the resource was found in memory, False if it
    Wasn't.  Use this to reduce the memory footprint, if you're sure you won't
    be needing the resource again.
    """</span>
    
    try:
        del __images[filename]
        return 1
    except KeyError:
        return 0


<span class="pydef">def get_font(filename, size, force_reload = 0):</span>
    <span class="pycomment">""" 
    get_font(filename, size, force_reload = 1) --> surface
    Call this function instead of pygame.font.Font - it will load the font
    from the disk the first time, then just return a reference to the copy each 
    subsequent time.
    """</span>
    if not __fonts_path:
        raise ValueError, "<span class="pystring">resources.set_fonts_path() not called yet.</span>"
    if (force_reload == 1 or filename not in __fonts.keys()):
        try:
            font = pygame.font.Font(os.path.join(__fonts_path, filename), size)
        except pygame.error:
            raise IOError, "<span class="pystring">File </span>" + filename + "<span class="pystring"> not found.</span>"
        __fonts[filename] = font
        return font
    else:
        return __fonts[filename]

<span class="pydef">def has_font(filename):</span>
    <span class="pycomment">"""
    has_font(filename) --> Boolean
    Returns true if the font is in memory, false if it has to be loaded from
    disk.
    """</span>
    return __fonts.has_key(filename)

<span class="pydef">def clear_font(filename):</span>
    <span class="pycomment">"""
    clear_font(filename) --> Boolean
    Eliminates the font from memory.  Subsequent calls will load it from the
    disk.  Returns True if the resource was found in memory, False if it
    Wasn't.  Use this to reduce the memory footprint, if you're sure you won't
    be needing the resource again.
    """</span>
    try:
        del __fonts[filename]
        return 1
    except KeyError:
        return 0

<span class="pydef">def get_sound(filename, force_reload = 0):</span>
    <span class="pycomment">"""
    get_sound(filename, force_reload = 1) --> sound
    Call this function instead of pygame.mixer.Sound - it will load the sound
    from the disk the first time, then just return a reference to the copy each 
    subsequent time.  
    """</span>
    if not __sounds_path:
        raise ValueError, "<span class="pystring">resources.set_sounds_path() not called yet.</span>"
    if (force_reload == 1 or filename not in __fonts.keys()):
        try:
            sound = pygame.mixer.Sound(os.path.join(__sounds_path, filename))
        except pygame.error:
            raise IOError, "<span class="pystring">File </span>" + filename + "<span class="pystring"> not found.</span>"
        __sounds[filename] = sound
        return sound
    else:
        return __sounds[filename]

<span class="pydef">def has_sound(filename):</span>
    <span class="pycomment">"""
    has_sound(filename) --> Boolean
    Returns true if the sound is in memory, false if it has to be loaded from
    disk.
    """</span>
    return __sounds.has_key(filename)

<span class="pydef">def clear_sound(filename):</span>
    <span class="pycomment">"""
    clear_sound(filename) --> Boolean
    Eliminates the sound from memory.  Subsequent calls will load it from the
    disk.  Returns True if the resource was found in memory, False if it
    Wasn't.  Use this to reduce the memory footprint, if you're sure you won't
    be needing the resource again.
    """</span>
    try:
        del __sounds[filename]
        return 1
    except KeyError:
        return 0

<span class="pydef">def set_images_path(path):</span>
    <span class="pycomment">"""
    set_images_path(path) --> Boolean
    Set the path you'll be loading the images off of.  Pass the string
    representation of the new path.  Raises an exception if the path doesn't
    exist, otherwise it returns True.
    """</span>
    if not os.access(path, os.F_OK):
        raise IOError, path + "<span class="pystring"> not found.</span>"
    if path.endswith(os.sep):
        path = path[:-1]
    global __images_path
    __images_path = path
    return 1

<span class="pydef">def set_fonts_path(path):</span>
    <span class="pycomment">"""
    set_fonts_path(path) --> Boolean
    Set the path you'll be loading the fonts off of.  Pass the string
    representation of the new path.  Raises an exception if the path doesn't
    exist, otherwise it returns True.
    """</span>
    if not os.access(path, os.F_OK):
        raise IOError, path + "<span class="pystring"> not found.</span>"
    if path.endswith(os.sep):
        path = path[:-1]
    global __fonts_path
    __fonts_path = path
    return 1

<span class="pydef">def set_sounds_path(path):</span>
    <span class="pycomment">"""
    set_sounds_path(path) --> Boolean
    Set the path you'll be loading the sounds off of.  Pass the string
    representation of the new path.  Raises an exception if the path doesn't
    exist, otherwise it returns True.
    """</span>
    if not os.access(path, os.F_OK):
        raise IOError, path + "<span class="pystring"> not found.</span>"
    if path.endswith(os.sep):
        path = path[:-1]
    global __sounds_path
    __sounds_path = path
    return 1

    
<span class="pydef">def get_images_path():</span>
    <span class="pycomment">"""
    get_images_path() --> String
    Returns the current value of the images path, or None if it hasn't been
    set yet.
    """</span>
    return __images_path

<span class="pydef">def get_fonts_path():</span>
    <span class="pycomment">"""
    get_fonts_path() --> String
    Returns the current value of the fonts path, or None if it hasn't been
    set yet.
    """</span>
    return __fonts_path

<span class="pydef">def get_sounds_path():</span>
    <span class="pycomment">"""
    get_sounds_path() --> String
    Returns the current value of the sounds path, or None if it hasn't been
    set yet.
    """</span>
    return __sounds_path

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

