<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Emboss / Cutout Text Effects</H3>
            <p>Author: Sherwyn Ramkissoon (bei at sympatico.ca)<br>
            Submission date: December 18, 2001</p>
	  
            <p>Description: A script that demonstrates several common image manipulation effects on a text string.</p>
            
            <p align="center"><img src="cutout.jpg" width="456" height="336"></p>

            <p>Download: <a href="cutout.py">cutout.py</a></p>	  

            <p>pygame version required: Any (with numeric)<br>
            SDL version required: Any<br>
            Python version required: Any</p>

            <p>Comments: This is a pretty terrifyingly complex script that uses Numeric to generate some very attractive filter effects on some text.  It's not actually a module, so you can't really import and reuse it, but the necessary fixes to allow that are pretty simple.  The overloading of the word 'filter' as a function name and argument doesn't necessarily conflict with the built-in python keyword 'filter', but reduces readability somewhat.  Still, very pretty.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment"># cutout.py
#
# Having recently discovered Python, this looked like a
# fun way to learn the language...
#
# Uses 5x5 convolution matrices to do Emboss, Cutout, Blur & Edge effects.
# Generates a somewhat random circular gradient texture.
#
# Even using Numpy this is pretty slow. 
# 
# Bits of code blatantly stolen from the Pygame site.
# 
# Author: Sherwyn Ramkissoon (bei@sympatico.ca)</span>


<span class="pycomment"># Import some useful (or not) stuff</span>
import os
import sys
import pygame
import pygame.image
import pygame.transform
import pygame.font
import Numeric as N
import math as math
import time as time
import random as random
import pygame.surfarray as surfarray
from pygame.locals import *


<span class="pycomment"># FILTER STUFF</span>
	
<span class="pycomment"># Convolution Kernels</span>
blurFilter = [[0,1,2,1,0],[1,2,4,2,1],[2,4,8,4,2],[1,2,4,2,1],[0,1,2,1,0]]
blurDiv = 48
cutoutFilter = [[-4,-3,-2,0,0],[-3,-2,-1,0,0],[-2,-1,18,1,2],[0,0,1,2,3],[0,0,2,3,4]]
embossFilter = [[4,3,2,0,0],[3,2,1,0,0],[2,1,18,-1,-2],[0,0,-1,-2,-3],[0,0,-2,-3,-4]]
cutDiv = 18
edgeFilter = [[0,0,0,0,0],[0,0,-1,0,0],[0,-1,5,-1,0],[0,0,-1,0,0],[0,0,0,0,0]]
edgeDiv = 1

<span class="pycomment"># 5x5 Convolution filter</span>
<span class="pydef">def Filter(inarray,filter,divisor):	</span>
	size = len(filter)	
	out = N.zeros(inarray.shape)
	(R,C,D) = out.shape
	for i in range((1-size)/2,(size+1)/2):
		for j in range((1-size)/2,(size+1)/2):
			if filter[i+2][j+2] != 0:
				out[2:R-2,2:C-2] += inarray[2+i:R-2+i,2+j:C-2+j]*filter[i+2][j+2]
	out = N.clip(out/divisor,0,255)
	return out

<span class="pycomment"># Blur Filter</span>
<span class="pydef">def Blur(inarray):</span>
	return Filter(inarray,blurFilter,blurDiv)

<span class="pycomment"># If font color is lighter than background, Cutout produces an embossed effect.</span>
<span class="pycomment"># If font color is darker than background, Cutout produces a cutout effect.</span>
<span class="pydef">def Cutout(inarray):</span>
	return Filter(inarray,cutoutFilter,cutDiv)

<span class="pycomment"># The opposite of Cutout </span>			
<span class="pydef">def Emboss(inarray):</span>
	return Filter(inarray,embossFilter,cutDiv)

<span class="pydef">def Edge(inarray):</span>
	return Filter(inarray,edgeFilter,edgeDiv)

<span class="pycomment"># Apply Filter to surface</span>
<span class="pydef">def Effect(surf,filterFunc):</span>
	pix = pygame.surfarray.array3d(surf)
	surf = pygame.Surface((pix.shape[0], pix.shape[1]), 32)
	surfarray.blit_array(surf,filterFunc(pix))
	return surf


<span class="pycomment"># GRADIENT STUFF</span>

<span class="pydef">def R(m):</span>
	return random.uniform(0,m)
	
<span class="pycomment"># Silly gradient function using polar coordinates</span>
<span class="pydef">def circGradient(rad,theta,freq,phase,noise):</span>
	g = int(math.fabs((255-noise)*math.cos(rad/50.0+phase)) + R(noise))
	r = int(math.fabs((255-noise)*math.cos(freq*theta+phase)) + R(noise))
	b = int(math.fabs((255-noise)*math.sin(freq*theta/2+phase)) + R(noise))
	return (r,g,b)
			
<span class="pycomment"># Gradient Generator - Generates some pixels & interpolates the rest</span>
<span class="pydef">def calcGradient(W,H,Cx,Cy,freq,phase,noise,step):</span>
	<span class="pycomment"># Screen seems to be in column-major order</span>
	bg = N.zeros((W,H,3))
	for i in range(0,W,step):
		xpos = i-W/2-Cx
		for j in range(0,H,step):
			ypos = j-H/2-Cy
			theta = math.atan2(ypos,xpos)
			rad = math.sqrt(xpos*xpos+ypos*ypos)
			bg[i][j] = circGradient(rad,theta,freq,phase,noise)
	<span class="pycomment"># Interpolate along y</span>
	ns = step
	while (ns > 1):
		ns2 = ns/2
		bg[0::step,ns2:-ns2:ns,:] = (bg[0::step,0:-ns:ns,:] + bg[0::step,ns::ns,:])/2		
		ns = ns2
	<span class="pycomment"># Interpolate along x</span>
	ns = step
	while (ns > 1):
		ns2 = ns/2
		bg[ns2:-ns2:ns,:,:] = (bg[0:-ns:ns,:,:] + bg[ns::ns,:,:])/2		
		ns = ns2		
	return bg

<span class="pycomment"># Couldn't get alpha to work, so simulate it (slowly) :-|</span>
<span class="pydef">def CalcAlpha(inarray):</span>
	inarray = calcGradient(inarray.shape[0],inarray.shape[1],50,0,5,0,80,8)
	return inarray

<span class="pycomment"># Apply pseudo-alpha gradient</span>
<span class="pydef">def PAlpha(surf):</span>
	pix = pygame.surfarray.array3d(surf)
	pix = N.array(pix,N.Int16)
	pAlpha = N.ones(pix.shape)*255
	pAlpha = CalcAlpha(pAlpha)
	pix = (pAlpha*pix)/255
	try:
 		pix = (pix*255)/N.argmax(N.argmax(N.argmax(pix)))
 	except:
 		pass
	surf = pygame.Surface((pix.shape[0], pix.shape[1]), 32)
	surfarray.blit_array(surf,pix)
	return surf
			

<span class="pycomment"># MAIN PROGRAM</span>
			
<span class="pycomment"># Setup	Pygame</span>
pygame.init()
font = pygame.font.Font(None, 90)
<span class="pycomment">#font = pygame.font.Font("c:/windows/fonts/stencil.ttf",80)</span>

<span class="pycomment"># Generate Text</span>
text = []
print "<span class="pystring">Thinking..</span>",
text.append(font.render("<span class="pystring">Emboss</span>",0,(220,120,120),(120,50,50)))
print "<span class="pystring">\b.</span>",
text.append(font.render("<span class="pystring">Cutout</span>",0,(20,60,120),(100,150,200)))
print "<span class="pystring">\b.</span>",
text.append(Effect(text[0],Cutout))
print "<span class="pystring">\b.</span>",
text.append(Effect(text[1],Cutout))
print "<span class="pystring">\b.</span>",
text.append(Effect(text[2],Blur))
print "<span class="pystring">\b.</span>",
text.append(Effect(text[3],Blur))
print "<span class="pystring">\b.</span>",
text.append(Effect(Effect(Effect(PAlpha(text[0]),Cutout),Blur),Edge))
print "<span class="pystring">\b.</span>",
text.append(Effect(Effect(Effect(PAlpha(text[1]),Cutout),Blur),Edge))
print "<span class="pystring">Done!</span>",

<span class="pycomment"># Setup screen</span>
H = text[0].get_height()*len(text)/2
W = text[0].get_width() + text[1].get_width()
screen = pygame.display.set_mode((W,H), 0, 32)
pygame.display.set_caption("<span class="pystring">Filter Effects</span>")

<span class="pycomment"># Display everything</span>
for i in range(len(text)):
	screen.blit(text[i], ((i%2)*text[0].get_width(),(int(i/2))*text[0].get_height()) )	
pygame.display.flip()


<span class="pycomment">#wait for the finish</span>
while 1:
    event = pygame.event.wait()
    if event.type is KEYDOWN and event.key == K_s: #save it
        name = os.path.splitext(sys.argv[0])[0] + '<span class="pystring">.bmp</span>'
        print '<span class="pystring">Saving image to:</span>', name
        pygame.image.save(screen, name)
    elif event.type in (QUIT,KEYDOWN,MOUSEBUTTONDOWN):
        break

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

