<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title: Graphical Cursor Class </H3>
            <p>Author: Frank Raiser (crashchaos at gmx.net) and Pete Shinners (pete at shinners.org)<br>
            Submission date: December 14, 2001</p>
	  
            <p>Description: This class replaces the standard cursor with a new version that gets its appearance from any surface (such as one returned by image.load()).  The graphical cursor responds to mousemotion events just like the normal cursor.</p>
            
            <p align="center"><img src="gfxcursor.gif" width="320" height="200"></p>

            <p>Download: <a href="gfxcursor.py">gfxcursor.py</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: Any</p>

            <p>Comments: The example script included with this class is pretty neat - it allows you to draw as in a paint program, over a multi-coloured background.  Note that the graphical cursor (a simple green circle in this case) reverts to the standard pygame cursor whenever drawing is taking place.  The idea of using a software-generated artificial cursor, rather than pygame's standard cursor, is an especially good one if you're considering using hardware acceleration (pygame.HWSURFACE) - the standard cursor may flicker or disappear altogether on a hardware surface.  It would be pretty trivial to subclass gfxcursor to make an animated cursor - just create an "animate()" method, and call that from update().  Also note that the cursor's constructor function takes a simple surface as an argument, so the image could be generated with pygame.draw functions as it is here, loaded from an image file, or even created with pygame.font.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""
This is a nice little GfxCursor class that gives you arbitrary mousecursor
loadable from all SDL_image supported filetypes. 

Author: Raiser, Frank aka CrashChaos (crashchaos at gmx.net)
Author: Shinners, Pete aka ShredWheat
Version: 2001-12-15

Usage:
Instantiate the GfxCursor class. Either pass the correct parameters to
the constructor or use setCursor, setHotspot and enable lateron.

The blitting is pretty optimized, the testing code at the bottom of
this script does a pretty thorough test of all the drawing cases.
It enables and disables the cursor, as well as uses a changing background.

In your mainloop, the cursor.show() should be what you draw last
(unless you want objects on top of the cursor?). Then before drawing
anything, be sure to call the hide(). You can likely call hide() immediately
after the display.flip() or display.update().

The show() method also returns a list of rectangles of what needs to be
updated. You can also move the cursor with pygame.mouse.set_pos()


That's it. Have fun with your new funky cursors.
"""</span>

import pygame

<span class="pydef">class GfxCursor:</span>
    <span class="pycomment">"""
    Replaces the normal pygame cursor with any bitmap cursor
    """</span>

    <span class="pydef">def __init__(self,surface,cursor=None,hotspot=(0,0)):</span>
        <span class="pycomment">"""
        surface = Global surface to draw on
        cursor  = surface of cursor (needs to be specified when enabled!)
        hotspot = the hotspot for your cursor
        """</span>
        self.surface = surface
        self.enabled = 0
        self.cursor  = None
        self.hotspot = hotspot
        self.bg      = None
        self.offset  = 0,0
        self.old_pos = 0,0
        
        if cursor:
            self.setCursor(cursor,hotspot)
            self.enable()

    <span class="pydef">def enable(self):</span>
        <span class="pycomment">"""
        Enable the GfxCursor (disable normal pygame cursor)
        """</span>
        if not self.cursor or self.enabled: return
        pygame.mouse.set_visible(0)
        self.enabled = 1

    <span class="pydef">def disable(self):</span>
        <span class="pycomment">"""
        Disable the GfxCursor (enable normal pygame cursor)
        """</span>
        if self.enabled:
            self.hide()
            pygame.mouse.set_visible(1)
            self.enabled = 0

    <span class="pydef">def setCursor(self,cursor,hotspot=(0,0)):</span>
        <span class="pycomment">"""
        Set a new cursor surface
        """</span>
        if not cursor: return
        self.cursor = cursor
        self.hide()
        self.show()
        self.offset = 0,0
        self.bg = pygame.Surface(self.cursor.get_size())
        pos = self.old_pos[0]-self.offset[0],self.old_pos[1]-self.offset[1]
        self.bg.blit(self.surface,(0,0),
            (pos[0],pos[1],self.cursor.get_width(),self.cursor.get_height()))

        self.offset = hotspot

    <span class="pydef">def setHotspot(self,pos):</span>
        <span class="pycomment">"""
        Set a new hotspot for the cursor
        """</span>
        self.hide()
        self.offset = pos

    <span class="pydef">def hide(self):</span>
        <span class="pycomment">"""
        Hide the cursor (useful for redraws)
        """</span>
        if self.bg and self.enabled:
            return self.surface.blit(self.bg,
                (self.old_pos[0]-self.offset[0],self.old_pos[1]-self.offset[1]))

    <span class="pydef">def show(self):</span>
        <span class="pycomment">"""
        Show the cursor again
        """</span>
        if self.bg and self.enabled:
            pos = self.old_pos[0]-self.offset[0],self.old_pos[1]-self.offset[1]
            self.bg.blit(self.surface,(0,0),
                (pos[0],pos[1],self.cursor.get_width(),self.cursor.get_height()))
            return self.surface.blit(self.cursor,pos)

    <span class="pydef">def update(self,event):</span>
        <span class="pycomment">"""
        Update the cursor with a MOUSEMOTION event
        """</span>
        self.old_pos = event.pos

if __name__ == '<span class="pystring">__main__</span>': <span class="pycomment">#test it out</span>
    import pygame.draw
    pygame.init()
    screen = pygame.display.set_mode((400, 300))
    screen.fill((50, 50, 111), (0, 0, 400, 150))
    pygame.display.flip()
    pygame.display.set_caption('<span class="pystring">Test the GfxCursor (and paint)</span>')
    
    image = pygame.Surface((20, 20))
    pygame.draw.circle(image, (50, 220, 100), (10, 10), 8, 0)
    pygame.draw.circle(image, (220, 200, 50), (10, 10), 8, 2)
    image.set_at((9, 9), (255,255,255))
    image.set_colorkey(0, pygame.RLEACCEL)
    
    magicbox = pygame.Rect(10, 10, 100, 90)
    magiccolor = 0
    
    cursor = GfxCursor(screen, image, (10, 10))
    finished = 0
    downpos = None
    while not finished:
        dirtyrects = []
        dirtyrects.extend([cursor.hide()])
        for e in pygame.event.get():
            if e.type in (pygame.QUIT, pygame.KEYDOWN):
                finished = 1
                break
            elif e.type == pygame.MOUSEBUTTONDOWN:
                cursor.disable()
                downpos = e.pos
            elif e.type == pygame.MOUSEBUTTONUP:
                cursor.enable()
                downpos = None
            elif downpos and e.type == pygame.MOUSEMOTION:
                r = pygame.draw.line(screen, (100,100,100), downpos, e.pos, 2)
                dirtyrects.append(r)
                downpos = e.pos
                cursor.update(e)
            elif not downpos and e.type == pygame.MOUSEMOTION:
                cursor.update(e)
        
        magiccolor = (magiccolor + 2) % 255
        r = screen.fill((0, 0, magiccolor), magicbox)
        dirtyrects.append(r)
        
        <span class="pycomment">#here's how we sandwich the flip/update with cursor show and hide</span>
        dirtyrects.extend([cursor.show()])
        pygame.display.update(dirtyrects)
        
        pygame.time.delay(5) <span class="pycomment">#should be time.wait(5) with pygame-1.3 :]</span>

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

