<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Interface to a high-score file</H3>
            <p>Author: Kevin Turner (acapnotic at users.sourceforge.net)<br>
            Submission date: April 13, 2001</p>
	  
            <p>Description: This provides an interface for the loading and saving the game's high score file.  High score files are generally placed in a location shared between all the players on that machine, but not writable by them (on filesystems which support file permissions).  For the game to be able to write to the score file, it must be run from a setgid wrapper.</p>

            <p>Download: <a href="scorefile.py">scorefile.py</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: 2.0</p>

            <p>Comments: This code pickles a machine-wide high-score listing to (by default) /var/games.  The high-score file contains username, realname, date, score/level, version and custom info fields, and should be created by your installation script.  In order to get around permissions issues under unix, scorefile.py will create a C source file for you, which you must then compile and install properly yourself - read the write_wrapper_source docstring carefully. This code should work with little modification on non-unix filesystems.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">#!/usr/bin/env python2
# $Id: scorefile.py,v 1.2 2001/04/13 20:15:09 kevint Exp $
# File authored by Kevin Turner (acapnotic at users.sourceforge.net)
# Created April 11, 2001
# This file is released into the Public Domain,
# but we always appreciate it if you send notice to myself or the pygame
# team (http://pygame.seul.org/) if you use it in your project or if
# you have questions, comments, or modifications.  And of course, if it
# brings you fantastic good fortune, we'd be happy to accept a token of
# your gratitude.  On the other hand, we make no guarantees (none at
# all), so if it ruins your life, don't blame us.

"""Interface to the high score file.

This provides an interface for the loading and saving the game's high
score file.  High score files are generally placed in a location shared
between all the players on that machine, but not writable by them (on
filesystems which support file permissions).  For the game to be able
to write to the score file, it must be run from a setgid wrapper.

All the fun is in the ScoreFile class.

There's also a write_wrapper_source subroutine, see its documentation
for information about the need for a set-group-id (SGID) wrapper.

NOTE: You should be warned that if run SGID, this module will change
the process EGID upon import."""</span>

TRUE = 1 == 1
FALSE = not TRUE

import os
import time
import pickle
import sys
from types import *

<span class="pycomment"># Try importing pygame, so we can define some rendering methods.</span>
try:
    import pygame
except ImportError:
    pass

<span class="pycomment"># According to the Filesystem Hierarchy Standard (2.2 beta),
# /var/games is the place to put score files.
# XXX - What's the default path for non-Unix systems?</span>
default_dir = os.path.normpath('<span class="pystring">/var/games</span>')
    
__methods__ = ['<span class="pystring">write_wrapper_source</span>']

<span class="pydef">class ScoreFileError:</span>
    <span class="pydef">def __init__(self, errorstring):</span>
        self.value = errorstring
    <span class="pydef">def __str__(self):</span>
        return `self.value`

<span class="pydef">class ScoreRecord:</span>
    <span class="pycomment">"""An individual record in the score file.

    score: A numeric value indicating the player's score.
    
    level: The number of the highest level or wave reached by the player.
           May be "None" if the game is not structured by level.

    login: The identity of this user, according to the OS,
           e.g. "agreensp".

    name: The name by which the player wishes to be known,
          e.g. "Alan Greenspan" or "MoneyMan".

    version: A string describing the version, or release, of the game
             software.

    date: The date and time when the score was recorded.  Stored in UTC,
          as a tuple.
    
    extradata: A dictionary object containing game-specific stats, e.g.
               board size, difficulty level, character race/class, etc.
               You may put anything in here you like, as long as it's
               pickleable.
    """ </span>             
    
    __members__ = ['<span class="pystring">date</span>','<span class="pystring">login</span>','<span class="pystring">name</span>',
                   '<span class="pystring">score</span>','<span class="pystring">level</span>', '<span class="pystring">extradata</span>']
    __methods__ = ['<span class="pystring">get_date</span>']

    <span class="pydef">def __init__(self, score, name=None, level=None, extradata=None):</span>
        self.date = time.gmtime(time.time())

        <span class="pycomment"># XXX - If we're a network game server, os.getlogin() probably
        # doesn't return what we want.</span>
        try:
            self.login = os.getlogin()
        except:
            self.login = None

        self.score = score
        self.name = name
        self.level = level

        self.extradata = extradata

    <span class="pydef">def get_date(self, formatstring=None):</span>
        <span class="pycomment">"""Returns a string describing the time of the entry, in local time.

        formatstring is the same format as used by time.strftime()."""</span>

        if not formatstring:
            formatstring = '<span class="pystring">%c</span>'

        formatstring = str(formatstring)

        <span class="pycomment"># self.date is a UTC time in a tuple.</span>
        utc_seconds = time.mktime(self.date)

        <span class="pycomment"># I thought I would have to test for time.daylight here,
        # and then use either time.altzone or time.timezone accordingly,
        # but apparently one of the other time functions already handles it,
        # because if I use altzone, I'm over-correcting by an hour.</span>
        local_offset = -time.timezone
            
        local_seconds = utc_seconds + local_offset

        local_time = time.localtime(local_seconds)

        time_string = time.strftime(formatstring, local_time)

        return time_string
        
    <span class="pydef">def __getattr__(self,name):</span>
        <span class="pycomment">"Check extradata to see if has this attribute."</span>
        if name != '<span class="pystring">extradata</span>' and self.extradata \
           and self.extradata.has_key(name):
            return self.extradata[name]
        raise AttributeError

    <span class="pydef">def __str__(self):</span>
        ds = {'<span class="pystring">score</span>':self.score,
              '<span class="pystring">name</span>':self.name,
              '<span class="pystring">level</span>':self.level,
              '<span class="pystring">login</span>':self.login,
              '<span class="pystring">date</span>':self.get_date()}
        
        if self.extradata:
            ds.update(self.extradata)

        return str(ds)
<span class="pycomment"># ScoreRecord</span>

<span class="pydef">class ScoreFile:</span>
    <span class="pycomment">"""A high score file which may be read from and added to.
    
    Entries are stored sorted by score."""</span>

    __members__ = ['<span class="pystring">filename</span>', '<span class="pystring">game_name</span>', '<span class="pystring">game_version</span>', '<span class="pystring">records</span>']
    __methods__ = ['<span class="pystring">add</span>', '<span class="pystring">write</span>', '<span class="pystring">crop</span>', '<span class="pystring">top</span>', '<span class="pystring">latest</span>']
    
    <span class="pycomment"># TODO: Provide a default DRAW method to render some scores to a SDL surface.</span>

    <span class="pydef">def __init__(self, game_name, game_version, filename=None, dirname=None):</span>
        <span class="pycomment">"""Load or create a score file.

        game_name: this name is stored in the score file, and possibly used to
                   generate a filename if none is supplied.
        game_version: this is the current version of the game, stored with each
                      entry.
        filename: the name of the high score file.  If none is given, an attempt
                  is made to construct one from game_name.
        dirname: the directory where the high score file should be placed, if
                 the filename is not specified as an absolute pathname.
                 The Filesystem Hierarchy Standard dictates that this be
                 /var/games.
        """</span>
        self.game_name = str(game_name)
        self.game_version = str(game_version)


        if dirname:
            dirname = os.path.normpath(dirname)
        else:
            dirname = default_dir
            
        if not os.path.abspath(dirname):
            dirname = os.path.join(default_dir,dirname)

        if filename:
            filename = os.path.normpath(filename)
        else:
            <span class="pycomment"># find a score file, or make a filename up (based on
            # game_name), or get one from the configuration module
            # XXX - How should this work on non-Unix platforms?
            # XXX - configuration module does not exist

            # XXX - filter game_name so it contains only
            #  filesystem-friendly characters.</span>
            filename = self.game_name + '<span class="pystring">.hiscore</span>'

        if not os.path.isabs(filename):
            filename = os.path.join(dirname, filename)

        <span class="pycomment"># This sort of thing *should* be done by the game's installer.
        # We'll make a bumbling attempt to try and get something working,
        # though.</span>
        if not os.path.exists(os.path.dirname(filename)):
            if _obtain_permission():
                os.makedirs(os.path.dirname(filename),0775)
            else:
                os.makedirs(os.path.dirname(filename))

        if not os.path.exists(filename):
            _obtain_permission()
            if os.access(os.path.dirname(filename), os.W_OK):
                <span class="pycomment"># we can create a scorefile, but for now it's empty.</span>
                self.records = []
            else:
                raise ScoreFileError, \
                      "<span class="pystring">I can't write to directory %s to create the </span>"\
                      "<span class="pystring">score file.</span>" % os.path.dirname(filename)
            _drop_permission()
        else:
            _obtain_permission()
            fobj = open(filename, '<span class="pystring">r</span>')
            try:
                loaded_scorefile = pickle.load(fobj)
                self.records = loaded_scorefile.records
                del loaded_scorefile
            except EOFError:
                <span class="pycomment"># We get this if the file is zero-sized.</span>
                self.records = []
            fobj.close()
            _drop_permission()

        self.filename = filename
    <span class="pycomment"># ScoreFile.__init__()</span>
    
    <span class="pydef">def add(self, score, name=None, level=None, extradata=None, write=TRUE):</span>
        <span class="pycomment">"""Make a new entry in the high score file.

        See ScoreRecord for a description of the arguments.
        Returns the entry's place in the high score list, with
        1 being the highest.
        
        write: Should the score file be written to disk after this add?
               If 'must', then raise an exception if the write fails.
               Otherwise, if true, attempt the write (but only print a
               warning if it fails), if false, add the entry to the score
               file in memory but make no attempt to save to disk.
        """</span>
        <span class="pycomment"># XXX - doc string should do necessary translcusion of ScoreRecord
        # argument docs, don't make the reader jump around to do our work.</span>
        
        score = float(score)
        name = str(name)

        score_rec = ScoreRecord(score, name, level, extradata)

        score_rec.game_version = self.game_version

        pos = self.place(score)
                    
        self.records.insert(pos-1, score_rec)

        if(write):
            if write=='<span class="pystring">must</span>':
                self.write()
            else:
                try:
                    self.write()
                except IOError:
                    print "<span class="pystring">WARNING: failed to save scorefile, got %s: %s</span>"\
                          % sys.exc_info()[:2]
        <span class="pycomment"># endif(write)</span>
        return pos
    <span class="pycomment"># ScoreFile.add()</span>

    <span class="pydef">def place(self,score):</span>
        <span class="pycomment">"""Returns the place that this score would earn in the list."""</span>
    
        if (not self.records) or score >= self.records[0].score:
            <span class="pycomment"># New High Score!</span>
            pos = 0
        elif score < self.records[-1].score:
            <span class="pycomment"># Congratz -- Yours is the lowest score yet.</span>
            pos = len(self.records)
        else:
            <span class="pycomment"># binary search to find the insertion location</span>
            high_end = len(self.records) - 1
            low_end = 0
            pos = 0
            while abs(high_end - low_end) > 1:
                pos = int(round((high_end + low_end) / 2))
                <span class="pycomment"># Reminder: A HIGHER score means a LOWER index.</span>
                if(score >= self.records[pos].score):
                    high_end = pos
                else:
                    low_end = pos
            <span class="pycomment"># whend</span>
        <span class="pycomment"># fi</span>
        return pos + 1
    <span class="pycomment"># ScoreFile.place()</span>

    <span class="pydef">def write(self):</span>
        <span class="pycomment">"""Save the score file to disk.

        Note: ScoreFile.add() calls this method, so you should
        rarely need to do so yourself."""</span>
        assert(self.filename)

        if _obtain_permission():
            <span class="pycomment"># Err.  We can't mask the write bit for the file's owner,
            # otherwise this user won't be able to write to it in the future!
            # Ideally, the file has already been created (by the install process),
            # and so is already owned by someone else.</span>
            os.umask(02)
        fobj = open(self.filename, '<span class="pystring">w</span>')
        pickle.dump(self, fobj)
        fobj.close()
        _drop_permission()

    <span class="pydef">def crop(self, bytes): # XXX - Untested!</span>
        <span class="pycomment">"""Limits the size of the scorefile.

        This method may be used to limit the size of the scorefile.
        It will remove as many entries from the scorefile as are
        required to make it fit in the specified number of bytes.
        The entries removed are those with the lowest scores.

        Returns the number of entries trimmed."""</span>

        bytes = long(bytes)
        assert(bytes > 0)

        trimmed_entries = 0

        while 1:
            pickledfile = pickle.dumps(self)        

            fat = len(pickledfile) - bytes
        
            if fat <= 0:
                return trimmed_entries
            else:
                if len(self.records) == 0:
                     raise ScoreFileError, "<span class="pystring">Failed to make the score </span>"\
                           "<span class="pystring">file small enough for you.</span>"
                del self.records[-1]
                trimmed_entries = trimmed_entries + 1
        <span class="pycomment"># whend</span>
    <span class="pycomment"># ScoreFile.crop()</span>

    <span class="pydef">def __get_item__(self, key):</span>
        <span class="pycomment">"""Scores are returned sorted highest to lowest."""</span>
        return self.records[key]

    <span class="pydef">def get_by_time(self, key):</span>
        <span class="pycomment">"""Like get_item, but sorted by time, not score."""</span>
        time_list = map(None, self.records[:].date, range(len(self.records)))

        time_list.sort()

        time_sorted = [ self.records[tl[1]] for tl in time_list ]

        return time_sorted[key]
    <span class="pycomment"># ScoreFile.get_by_time()</span>

    <span class="pydef">def top(self, n=1):</span>
        <span class="pycomment">"""Returns a list of the top n scores (defaulting to 1).

        Returns a list of ScoreRecords.
        Note: This object supports the standard slice syntax,
        so this method is equivalent to my_scorefile[:n]."""</span>

        return self[:n]

    <span class="pydef">def latest(self, n=1):</span>
        <span class="pycomment">"""Returns the n most recent entries in the score file.

        If n is omitted, it defaults to 1.
        Returns a list of ScoreRecords, starting with the most recent."""</span>

        results = self.get_by_time(-n)
        results.reverse()
        return results

    <span class="pydef">def __len__(self):</span>
        <span class="pycomment">"""The number of entries in the high score file."""</span>
        return len(self.records)

    <span class="pydef">def __str__(self):</span>
        s = "<span class="pystring">%s: Score File for %s, contains %d entries.\n</span>" %\
            (self.filename, self.game_name, len(self.records))
        for i in self.records:
            sr = "<span class="pystring">%d\t%s\t%s\t%s\t%s</span>" \
                 % (i.score, i.level, i.name, i.login, i.get_date())
            if i.extradata:
                sr = sr + str(i.extradata)
            s = s + sr + "<span class="pystring">\n</span>"
        return s
    <span class="pycomment"># ScoreFile.__str__()</span>
<span class="pycomment"># ScoreFile</span>


<span class="pydef">def _obtain_permission():</span>
    <span class="pycomment">"""Re-gain our SGID group identity."""</span>
    if _SGID_WRAPPER:
        os.setregid(_user_gid,_privledged_gid)
        return TRUE
    return FALSE

<span class="pydef">def _drop_permission():</span>
    <span class="pycomment">"""Drop our SGID group identity for the time being."""</span>
    if _SGID_WRAPPER:
        os.setregid(_privledged_gid,_user_gid)
        return TRUE
    return FALSE

<span class="pycomment"># top-level code</span>
if os.environ.has_key('<span class="pystring">RUN_BY_WRAPPER</span>'):
    _privledged_gid = os.getegid()
    _user_gid = os.getgid()
    if _privledged_gid != _user_gid:
        _SGID_WRAPPER = TRUE
    else:
        _SGID_WRAPPER = FALSE
        print "<span class="pystring">Apparently run from a wrapper, but it doesn't seem to be SGID.</span>"    
    <span class="pycomment"># XXX: This causes us to drop our SGID permissions upon importing
    # the module!  If our parent module was not expecting those to
    # change, it may not be happy with us.</span>
    _drop_permission()
else:
    _SGID_WRAPPER = FALSE
    print "<span class="pystring">Warning: Not run from a SGID wrapper, I may use inappropriate</span>"
    print "<span class="pystring"> ownership and/or permissions for the high score file.</span>"
    <span class="pycomment"># TODO:  At this point, if we believe there *should* be a wrapper installed,
    #  but we just weren't run by it, we could exec it here, if we wanted to.</span>
<span class="pycomment"># top-level code</span>

<span class="pydef">def write_wrapper_source(path,source_file):</span>
    <span class="pycomment">"""Writes C source code that exec's path to source_file.

    On some platforms (e.g. unix) it's desirable to have a shared score
    file between users.  This requires that the game be able to write to
    the score file no matter who's playing, but we'd rather not make the
    file world-writable.

    If the score file is installed as owned by a "games" group, then
    the game could write to that file if it were owned by the games
    group and had its sgid bit set (chmod g+s).  Unfortunately for
    us, most systems don't use sgid for interpreted files (e.g. your
    python script), so we need to make a "real executable" to put the
    sgid bit on.  This, having gained the "games" permissions, will
    in turn run our python script.

    This function writes the source for such a wrapper program to
    a file.  You must then see to it that the file is compiled and
    properly installed."""</span>
    
    path = os.path.normpath(path)
    source_file = os.path.normpath(source_file)

    if not os.path.exists(path):
        print "<span class="pystring">Warning: path</span>",path,"<span class="pystring">doesn't seem to exist.</span>"

    fobj = open(source_file, '<span class="pystring">w</span>')

    s = '<span class="pystring">#include &lt;unistd.h&gt;\n#include &lt;stdlib.h&gt;\n</span>'
    s = s + "<span class="pystring">/* Compile like: gcc -o my_wrapper %s */\n</span>" % source_file
    s = s + """<span class="pystring">
            int main(int argc, char* argv[])
            {
            putenv("RUN_BY_WRAPPER=1");</span>
            """
    fobj.write(s)
    fobj.write("<span class="pystring">return execv(\</span>"%s\"<span class="pystring">,argv);\n}\n</span>" % path)
    fobj.close()
<span class="pycomment"># write_wrapper_source()</span>

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

