<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Hollow/Outline text module</H3>
            <p>Author: Pete Shinners (Pete at shinners.org)<br>
            Submission date: December 03, 2001</p>
	  
            <p>Description: Two functions that create opaque or transparent text with a 1-pixel wide border around it.</p>
            <p align="center"><img src="hollow.gif" width="562" height="116"></p>

            <p>Download: <a href="hollow.py">hollow.py</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: Any</p>

            <p>Comments: TextHollow's fontcolor argument isn't actually the color of the text - this effect makes the text transparent.  Instead, it's the color of the outline.  Both functions return surfaces with the colorkey set; the supplied example script demonstrates this by blitting both over a varied background.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">#!/usr/bin/env python</span>

<span class="pycomment">"""
two text rendering styles. outlined and hollow. both
use a single pixel border around the text. you might be
able to cheat a bigger border by fooling with it some.
"""</span>
import os, sys, pygame, pygame.font, pygame.image
from pygame.locals import *


<span class="pydef">def textHollow(font, message, fontcolor):</span>
    notcolor = [c^0xFF for c in fontcolor]
    base = font.render(message, 0, fontcolor, notcolor)
    size = base.get_width() + 2, base.get_height() + 2
    img = pygame.Surface(size, 16)
    img.fill(notcolor)
    base.set_colorkey(0)
    img.blit(base, (0, 0))
    img.blit(base, (2, 0))
    img.blit(base, (0, 2))
    img.blit(base, (2, 2))
    base.set_colorkey(0)
    base.set_palette_at(1, notcolor)
    img.blit(base, (1, 1))
    img.set_colorkey(notcolor)
    return img

<span class="pydef">def textOutline(font, message, fontcolor, outlinecolor):</span>
    base = font.render(message, 0, fontcolor)
    outline = textHollow(font, message, outlinecolor)
    img = pygame.Surface(outline.get_size(), 16)
    img.blit(base, (1, 1))
    img.blit(outline, (0, 0))
    img.set_colorkey(0)
    return img


entry_info1 = '<span class="pystring">Hollow, by Pete Shinners</span>'
entry_info2 = '<span class="pystring">Outlined, by Pete Shinners</span>'

<span class="pycomment">#this code will display our work, if the script is run...</span>
if __name__ == '<span class="pystring">__main__</span>':
    pygame.init()

    <span class="pycomment">#create our fancy text</span>
    white = 255, 255, 255
    grey = 100, 100, 100
    bigfont = pygame.font.Font(None, 60)
    text1 = textHollow(bigfont, entry_info1, white)
    text2 = textOutline(bigfont, entry_info2, grey, white)

    <span class="pycomment">#create a window the correct size</span>
    width = max(text1.get_width(), text2.get_width())
    height = text1.get_height() + text2.get_height()
    win = pygame.display.set_mode((width, height))
    win.fill((20, 20, 80), (0, 0, width, 30))
    win.fill((20, 20, 80), (0, height-30, width, 30))

    win.blit(text1, (0, 0))
    win.blit(text2, (0, text1.get_height()))
    pygame.display.flip()
    
    <span class="pycomment">#wait for the finish</span>
    while 1:
        event = pygame.event.wait()
        if event.type is KEYDOWN and event.key == K_s: <span class="pycomment">#save it</span>
            name = os.path.splitext(sys.argv[0])[0] + '<span class="pystring">.bmp</span>'
            print '<span class="pystring">Saving image to:</span>', name
            pygame.image.save(win, name)
        elif event.type in (QUIT,KEYDOWN,MOUSEBUTTONDOWN):
            break

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

