<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Intersection Bobs</H3>
            <p>Author: Diez B. Roggisch (deets at web.de)<br>
            Submission date: September 22, 2002</p>
	  
            <p>Description: Simple intersection bob example, plus usage of invalidate rects to restore background.</p>
            <p align="center"><img src="intersection_bobs.png" width="320" height="240"></p>

            <p>Download: <a href="intersection_bobs.tgz">intersection_bobs.tgz</a></p>	  

            <p>pygame version required: Any with Surfarray<br>
            SDL version required: Any<br>
            Python version required: 2.0 with Numeric</p>

            <p>Comments: A demo effect from the early 90ties, intersection bobs combined <a href="http://www.pygame.org/pcr/vector_bob/index.php">vector bobs</a> with precalculated z-buffers. The results can be very impressive. A extension could be to use an animated bob with a different shape than a sphere.</p>
            <p>The archive file includes the POV files used to create the sphere used for the bob source graphics.</p>

            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""
A demo effect from the early 90ties, intersection bobs combined vector bobs with
precalculated z-buffers. The results can be very impressive. A extension could be to
use a animated bob with a different shape than a sphere.
"""</span>

try:
	import sys, types, pygame, pygame.image, pygame.display, pygame.rect, pygame.key
	import profile
	from Numeric import *
	
except ImportError, e:
	print "<span class="pystring">import exception occured</span>"
	print e
	sys.exit()


MODE = (640, 480)
BACKGROUND = None
BALL = None

<span class="pydef">class ZSprite:</span>
	<span class="pycomment">""" A ZSprite combines a surface and a hight-mask. The latter one should be supplied as
	a grey-image with values from 0..255. 0 means that the sprite doesn't use that pixel at all,
	255 is a pixel with the mininal z coordinate (aka closest to the viewer).
	"""</span>

	<span class="pycomment"># The bias is used to ease calculations. This insures that the</span>
	<span class="pycomment"># crucial zero never appears as coordinate.</span>
	BIAS = 2 ** 24
	<span class="pydef">def __init__(_, surface, z_surface):</span>
		if surface.get_size() != z_surface.get_size():
			raise "<span class="pystring">surface and z_surface dimensions differ!</span>"
		_.surface = surface
		_.createZArray(z_surface)

	<span class="pydef">def createZArray(_, z_surface):</span>
		_.z_array = asarray(pygame.surfarray.array3d(z_surface)[:,:,1], Int32)
		_.z_mask = greater(_.z_array, 0)
		
	<span class="pydef">def __add__(_, z):</span>
		return ((_.z_array + z) + ZSprite.BIAS) * _.z_mask
	
	<span class="pydef">def __sub__(_, z):</span>
		return ((_.z_array - z) + ZSprite.BIAS) * _.z_mask 

<span class="pydef">class BobObject:</span>
	<span class="pycomment">""" A BobObject. The list _.coords are the coordinates of the points to be blitted """</span>
	<span class="pydef">def __init__(_, zsprite, scadjust):</span>
		<span class="pycomment">""" surface - the points are blitted as that
		scadjust - a tuple representing the screen (width/2, height/2) to center the transformed coordinates
		"""</span>
		_.surface = zsprite.surface
		_.zsprite = zsprite
		_.coords = []
		_.scadjust = scadjust - array(_.surface.get_rect().size) / 2
		_.position = array((0,0,0))
		_.reset()
		_.colObs = []
		
	<span class="pydef">def translate(_, trans):</span>
		_.position += trans

	<span class="pydef">def reset(_):</span>
		_.orientation = array(identity(3), Float)
		
	<span class="pydef">def rotateZ(_, deg):</span>
		rMat = array(identity(3), Float)
		cos = math.cos(deg)
		sin = math.sin(deg)
		rMat[0][0] = cos
		rMat[0][1] = -sin
		rMat[1][0] = sin
		rMat[1][1] = cos
		_.orientation = matrixmultiply(_.orientation, rMat)

	<span class="pydef">def rotateY(_, deg):</span>
		rMat = array(identity(3), Float)
		cos = math.cos(deg)
		sin = math.sin(deg)
		rMat[0][0] = cos
		rMat[0][2] = -sin
		rMat[2][0] = sin
		rMat[2][2] = cos
		_.orientation = matrixmultiply(_.orientation, rMat)
		_.rot = deg

	<span class="pydef">def rotateX(_, deg):</span>
		rMat = array(identity(3), Float)
		cos = math.cos(deg)
		sin = math.sin(deg)
		rMat[1][1] = cos
		rMat[1][2] = -sin
		rMat[2][1] = sin
		rMat[2][2] = cos
		_.orientation = matrixmultiply(_.orientation, rMat)

	<span class="pydef">def screenCoords(_):</span>
		<span class="pycomment">""" This methods transforms and projects the coordinates of the bobs.
		"""</span>
		sc = []
		_.colObs = []
		for c in _.coords:
			p = matrixmultiply(c , _.orientation) + _.position
			sc.append((p[2], int(p[0] * 1000 / p[2] + _.scadjust[0]), int(p[1] * 1000 / p[2] + _.scadjust[1]), int(p[2])))

		<span class="pycomment">#sc.sort()</span>
		<span class="pycomment">#sc.reverse()</span>
		sc = map(lambda x: x[1:], sc)
		left = reduce(lambda x,y:(y,x)[x[0] &lt; y[0]], sc)
		right = reduce(lambda x,y:(y,x)[x[0] &gt; y[0]], sc)
		top = reduce(lambda x,y:(y,x)[x[1] &lt; y[1]], sc)
		bottom = reduce(lambda x,y:(y,x)[x[1] &gt; y[1]], sc)
		size = (int(right[0] - left[0]), int(bottom[1] - top[1]))
		<span class="pycomment"># The transformed screencoords</span>
		_.screen_coords = sc
		<span class="pycomment"># The extend of all sprites together.</span>
		_.size = size
		_.topleft = (left[0], top[1])

	<span class="pydef">def calcSurfaces(_):</span>
		<span class="pycomment">""" This is the primary method: here we calculate the individual masks for the bobs.
		"""</span>
		_.surfaces = []
		<span class="pycomment"># The back buffer is zero-initialized</span>
		ssz =_.zsprite.surface.get_size()
		back = zeros((_.size[0] + 16 + ssz[0], _.size[1] + 16 + ssz[1]))
		for i in xrange(len(_.coords)):
			<span class="pycomment"># tl is the coordinate of the bob relative to the</span>
			<span class="pycomment"># back buffer</span>
			tl = _.screen_coords[i][0] - _.topleft[0] + 8, _.screen_coords[i][1] - _.topleft[1] + 8
			sz = _.zsprite.surface.get_size()
			br = tl[0] + sz[0], tl[1] + sz[1]
			<span class="pycomment"># Get the zmap for the child</span>
			zmap = _.zsprite - _.screen_coords[i][2]
			s1 = slice(tl[0],br[0])
			s2 = slice(tl[1],br[1])
			<span class="pycomment"># The part of the back this sprite is going to cover</span>
			portion = back[s1, s2]
			diff = zmap - portion
			<span class="pycomment"># Every pixel where the current bob has a smaller z coordinate</span>
			<span class="pycomment"># will be set by it, the rest is cleared.</span>
			mask = asarray(greater(diff, 0), Int32)			
						
			<span class="pycomment"># Propagate the values in the back map</span>
			back[s1, s2] = (portion + (diff * mask)).astype(Int32)
			<span class="pycomment"># This makes the mask a valid alpha channel</span>
			mask *=255
			mask = mask.astype(UnsignedInt8)
			<span class="pycomment"># Create a new surface</span>
			sf = _.zsprite.surface.convert_alpha()
			sa = pygame.surfarray.pixels_alpha(sf)
			<span class="pycomment"># Apply the mask</span>
			sa[:,:] = mask
			_.surfaces.append(sf)
			
	<span class="pydef">def getBobs(_):</span>
		<span class="pycomment">""" Combine coordinates and computed images
		"""</span>
		_.screenCoords()
		_.calcSurfaces()
		bobs = []
		for i in xrange(len(_.coords)):		
			bobs.append((_.screen_coords[i][:-1], _.surfaces[i]))
		return bobs
		
<span class="pydef">class Sphere(BobObject):</span>
	<span class="pydef">def __init__(_, dim, surface, scad, segments = 8):</span>
		BobObject.__init__(_, surface, scad)
		nums = [(1, 0)]
		for n in xrange(1, segments >> 1):
			deg = n / float(segments >> 1) * math.pi / 2
			radius = dim * cos(deg)
			offset = dim * sin(deg)
			num = floor(segments * cos(deg))
			if int(num) & 1:
				num += 1
			for i in xrange(num):
				x = radius * cos(math.pi * i / num * 2)
				y = radius * sin(math.pi * i / num * 2)
				_.coords.append((x, y, offset))
				_.coords.append((x, y, -offset))

		for i in xrange(segments):
			x = dim * cos(math.pi * i / segments * 2)
			y = dim * sin(math.pi * i / segments * 2)
			_.coords.append((x, y, 0))
		_.coords.append((0, 0, -dim))
		_.coords.append((0, 0, dim))

<span class="pydef">class Ring(BobObject):</span>
	<span class="pydef">def __init__(_, dim, surface, scad, segments = 8):</span>
		BobObject.__init__(_, surface, scad)
		for i in xrange(segments):
			x = dim * cos(math.pi * i / segments * 2)
			y = dim * sin(math.pi * i / segments * 2)
			_.coords.append((x, y, 0))

<span class="pydef">class DisplayList:</span>
	<span class="pydef">def __init__(_):</span>
		_.saveStack = []
		_.bobs = []
		_.display = pygame.display.get_surface()
		
	<span class="pydef">def addBob(_, bob):</span>
		<span class="pycomment"># Create the rect in the display which has to be saved</span>
		rect = bob[1].get_rect().move(bob[0])
		bgSave = pygame.Surface(rect[2:])
		bgSave.blit(_.display, (0,0), rect)
		_.saveStack.append((bob[0], bgSave, rect))
		_.bobs.append(bob)

	<span class="pydef">def blitBobs(_):</span>
		rectList = []
		for bob in _.bobs:
			rectList.append(bob[1].get_rect().move(bob[0]))
			_.display.blit(bob[1], bob[0])
		_.bobs = []
		return rectList
	
	<span class="pydef">def restoreBackground(_):</span>
		rectList = []
		count = 0
		y = 48
		while len(_.saveStack):
			resBob = _.saveStack.pop()			
			rectList.append(resBob[2])			
			_.display.blit(resBob[1], resBob[0])
			count += 1
		return rectList
	
		
<span class="pydef">def main():</span>
	global DISPLAY
	global SCREENRECT
	global LEVEL
	global DIST_DEC
	DIST_DEC = 1
	pygame.init()
	pygame.display.init()

	rect = (0, 0) + MODE
	DISPLAY = pygame.display.set_mode(MODE, 0, 32)
	sphere_image = pygame.image.load("<span class="pystring">sphere.png</span>").convert_alpha()	
	SPHERE = ZSprite(sphere_image, pygame.image.load("<span class="pystring</span>">spherePP.png"))

	
	dl = DisplayList()
	sc = array(DISPLAY.get_rect()[2:])*0.5

	vBob = Sphere(200, SPHERE, sc, 6)
	vBob.translate(array((0, 0, 10009)))

	pygame.display.update()
	oldts = pygame.time.get_ticks() -100
	speed = 50.0
	at = 0
	r = 1.0

	while 1==1:
		ts = pygame.time.get_ticks()
		elapsed = ts - oldts
		oldts = ts
		pygame.event.pump()		
		keystate = pygame.key.get_pressed()
		if keystate[pygame.K_ESCAPE]:
			break

		invalidRects = dl.restoreBackground()
		vBob.reset()
		r = r + speed * elapsed / 1000
		vBob.rotateX((r / 180.0) * math.pi)
		vBob.rotateY((r / 180.0) * math.pi)

		bobs = vBob.getBobs()
		for b in bobs:
			dl.addBob(b)
		invalidRects += dl.blitBobs()
		pygame.display.update(invalidRects)		
		pygame.time.wait(50)

		pygame.display.update()		
		pygame.time.wait(50)

if __name__ == "<span class="pystring">__main__</span>":
	main()
	<span class="pycomment">#profile.run('main()')</span>

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

