<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title: Function to Create Cursors from Graphics Files </H3>
            <p>Author: Ned Batchelder (ned at nedbatchelder.com)<br>
            Submission date: November 24, 2001</p>
	  
            <p>Description: Creates pygame cursors from a graphic file.  This function allows you to draw your custom cursors in an image-editing program, and keep them all in a single file.</p>
            
            <p align="center"><img src="cursors.gif" width="64" height="64"></p>

            <p>Download: <a href="MakeCursors.zip">MakeCursors.zip</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: 2.0 or greater</p>

            <p>Comments: Your image file must be a multiple of 32x32 in size, and must use certain colors: Black (0,0,0) and White (255,255,255) are treated normally, Red (255, 0, 0) and Magenta (255, 0, 255) are used to specify the hotspot, and any other color (green in the example) is transparent.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""
Make pygame cursors from an image file
"""</span>

import pygame
import pygame.image
import pygame.cursors
    
<span class="pydef">def MakeCursors(filename, names):</span>
    <span class="pycomment">"""
    MakeCursors(filename, names):
    Read the image at filename, creating 32x32 cursors.  Return a dictionary
    of cursors, assigned to the names in names.
    Colors in the image:
    black and white are black and white.
    Red and magenta are the hotspot (colored black or white, respectively).
    Any other color is transparent.
    """</span>

    <span class="pycomment"># Load the raw tile image</span>
    try:
        tiles = pygame.image.load(filename)
    except pygame.error:
        raise pygame.error, '<span class="pystring">Could not load image "%s" %s</span>' % (filename, pygame.get_error())

    tileWidth = 32
    tileHeight = 32
    tileSize = (tileWidth, tileHeight)

    curs = {}

    tiles.lock()

    <span class="pycomment"># Split it into individual tiles</span>
    for ycur in range(tiles.get_height() / tileHeight):
        for xcur in range(tiles.get_width() / tileWidth):
            tileX = xcur * tileWidth
            tileY = ycur * tileHeight

            strings = []
            hot = (0,0)

            if len(names) > 0 and names[0] != '':            
                for y in range(tileY, tileY + tileHeight):
                    l = ''
                    for x in range(tileX, tileX + tileWidth):
                        clr = tiles.get_at((x, y))[:3]
                        <span class="pycomment"># not sure why, but the colors are impure.  Force them to black, white, or green</span>
                        pclr = map(lambda x: x > 100, clr)
                        if pclr == [0,0,0]:
                            l += '<span class="pystring">X</span>'
                        elif pclr == [1,1,1]:
                            l += '<span class="pystring">.</span>'
                        elif pclr == [1,0,0]:
                            l += '<span class="pystring">X</span>'
                            hot = (x - tileX, y - tileY)
                        elif pclr == [1,0,1]:
                            l += '<span class="pystring">.</span>'
                            hot = (x - tileX, y - tileY)
                        else:
                            l += '<span class="pystring"> </span>'
                    strings.append(l)

                data = pygame.cursors.compile(strings, "<span class="pystring">.</span>", "<span class="pystring">X</span>")
                curs[names[0]] = ( (tileWidth, tileHeight), hot) + data

            names = names[1:]

    tiles.unlock()
    
    return curs


if __name__ == '<span class="pystring">__main__</span>':
    pygame.init()
    print MakeCursors('<span class="pystring">cursors.gif</span>', ('<span class="pystring">arrow</span>', '<span class="pystring">forward</span>', '<span class="pystring">left</span>', '<span class="pystring">right</span>'))

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

