<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Mosaic Transition</H3>
            <p>Author: Gareth Noyce (korruptor at mac.com)<br>
            Submission date: June 29, 2002</p>
	  
            <p>Description: A fading, pixellizing mosaic transition effect.</p>
            
            <p align="center"><img src="mosaic1.jpg" width="400" height="300">
                <img src="mosaic2.png" width="400" height="300">
            </p>

            <p>Download: <a href="mosaic-transition.zip">mosaic-transition.zip</a></p>	  

            <p>pygame version required: 1.3.4<br>
            SDL version required: Any<br>
            Python version required: Python 2 (with Numeric)</p>

            <p>Comments: Here's another nice transition effect from Gareth Noyce. This one pixellates and fades a given image, and then restores it. You'll need surfarray, and a reasonably current version of pygame.  The supplied script demonstrates the effect on a nice sample image, by fading and restoring the image in a loop. Just hit a key or a mouse button to quit.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""
Program		:	mosaic.py
Author		:	Gareth "Korruptor" Noyce, http://www.korruptor.demon.co.uk
Description	:

This is a stupidly simple mosaic transition for a surface, essentially the same as the
transition Nintendo used in Super Mario World Advance 2 (Super Mario World on the SNES).

I've been stuck in traffic for 45mins, and I've been so bored that this is the result. :)

The mosaic function takes a destination surface, a source surfarray and a value for the size 
of each "pixel" it creates in the destination. The result is a "chunky" version of the source.

This version works on an 8bit palette, but it's trivial to convert to true colour... You may 
also want to precalc the first two frames to avoid the "stutter" that happens on largish
surfaces.

"""</span>

import pygame, pygame.image
from pygame.surfarray import *
from pygame.locals import *
from Numeric import *

<span class="pycomment"># ------------------------------------------------------------------------------------</span>

RES 		= (400,300)
PALETTE 	= zeros((256,3))
FADE_STEP	= zeros((256,3))
FRAMES          = 16

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def main():</span>
     
    	<span class="pycomment"># Initialise pygame, and grab an 8bit display.</span>
    	pygame.init()
    	screen		= pygame.display.set_mode(RES,0, 8)

	<span class="pycomment"># Load a dummy image and copy it to a surfarray</span>
    	image 		= pygame.image.load("<span class="pystring">pygame.gif</span>")
        image_buf 	= pygame.surfarray.array2d(image)  
        
        <span class="pycomment"># This will be the initial size of the mosaic pixels...</span>
        pixel_chunk = 2
        dir = 1

        <span class="pycomment"># Fruity loops...</span>
        while 1:
    
            <span class="pycomment"># Have we received an event to quit the program?</span>
            for e in pygame.event.get():
                if e.type in (QUIT,KEYDOWN,MOUSEBUTTONDOWN):
                    return

            <span class="pycomment"># Get the image palette</span>
            PALETTE = image.get_palette()

            <span class="pycomment"># Blit the original image...</span>
            screen.set_palette(PALETTE)
            screen.blit(image,(0,0))
            pygame.display.flip()

            <span class="pycomment"># Calculate the amounts to inc/dec each colour per frame
            # so we can fade to black. You could fade each colour by
            # a set amount, but this will ensure all colours hit black 
            # at the same time (looks a bit better imo)</span>
            for i in range(0,256):
                FADE_STEP[i][0] = int(PALETTE[i][0] / FRAMES) 
                FADE_STEP[i][1] = int(PALETTE[i][1] / FRAMES)
                FADE_STEP[i][2] = int(PALETTE[i][2] / FRAMES)

            <span class="pycomment"># Fade out</span>
            for i in range(0,FRAMES):
                pixelate(screen, image_buf, pixel_chunk)
                PALETTE -= FADE_STEP
                screen.set_palette(PALETTE)
                pygame.display.flip()
                pixel_chunk +=2

            <span class="pycomment"># Fade in</span>
            for i in range(0,FRAMES):
                pixelate(screen, image_buf, pixel_chunk)
                PALETTE += FADE_STEP
                screen.set_palette(PALETTE)
                pygame.display.flip()
                pixel_chunk -=2

        <span class="pycomment"># Bye!</span>
        return

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def pixelate(dest, src, size):</span>
    <span class="pycomment">"""
    Generates a mosaic of the src image to the dest surface. Size dictates the pixel size

    - Note: you should really average out the colour values of all pixels from 
      x,y to x+size,y+size for accuracy. This is just a quick hack, and as I'm fading the 
      palette...
    """</span>

    for y in range(0,RES[1],size):
        for x in range(0,RES[0],size):
            rtile 	= pygame.Rect([x,y,size,size])
            colour 	= src[x][y]
            dest.fill(colour, rtile)


<span class="pycomment"># ------------------------------------------------------------------------------------</span>
if __name__ == '<span class="pystring">__main__</span>': main()
<span class="pycomment"># End of sauce. Pass the chips...</span>

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

