<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  RGB Plasma Effect</H3>
            <p>Author: Gareth Noyce (g at korruptor.demon.co.uk)<br>
            Submission date: January 18, 2002</p>
	  
            <p>Description: An animated 'plasma' effect</p>
            
            <p align="center"><img src="plasma.png" width="320" height="256"></p>

            <p>Download: <a href="pygame-plasma.zip">pygame-plasma.zip</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: 2.0 or newer; Numeric required</p>

            <p>Comments: The plasma effect is a staple of the demo scene.  It's useful as a seed graphic for a variety of image effects, and is also a pretty effect by itself.  Mr. Noyce demonstrates a couple of nice techniques here - he precalculates the cosine values for the unit circle and stores the values in a table.  Caching math module functions like this can result in substantial speed gains in realtime effects.  Code that uses Numeric is usually fairly opaque and difficult to follow; luckily, Mr. Noyce has commented his functions quite well.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""Plasma.py -- Korruptor Jan 2001, test code for Seal Basher (v1.0)

This little doobry is based on the fast flame example by Pete Shinners
and is another novel neat hack on the surfarray module. 

The plasma algo itself is pretty simple, just a sum of four cosine values
from a pre-calculated look-up table inserted into a surf buff. It's all 
pretty easy really. The comments explain my thinking... 

This is my first hack, and not really optimised apart from what I've learnt
from Pete's example and whilst lurking on #Pygame. If you've got suggestions
for speed increases let me know..."""</span>

import pygame, pygame.transform
from math import *
from operator import *
from pygame.surfarray import *
from pygame.locals import *
from Numeric import *
from RandomArray import *

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pycomment"># Glob decs</span>

<span class="pycomment"># Screen resolution...</span>
RES 	= array((320,256))

<span class="pycomment"># That all important math thingy...</span>
PI 	= 3.14159

<span class="pycomment"># Linear array of cosine values to be indexed and summed, initialised to zero prior to pre-calc...</span>
cos_tab = zeros(256)	

<span class="pycomment"># Array of indexes to be used on our cos_tab. Could be variables I suppose. Just easier to cut_n_paste! ;-)</span>
pnt_tab = array((0,0,0,0))

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def main():</span>
    <span class="pycomment">"Inisalises display, precalculates the cosine values, and controls the update loop"</span>
    
    <span class="pycomment"># Initialise pygame, and grab an 8bit display.</span>
    pygame.init()
    screen_surface = pygame.display.set_mode(RES, 0, 8)

    <span class="pycomment"># Numeric array (working) for the display. Do all our fun stuff here...</span>
    plasma_buffer  = zeros(RES/8)
    
    <span class="pycomment"># Pygame Surface object which will take the surfarray data and be translated into a screen blit... </span>
    plasma_surface = pygame.Surface((RES[0]/8, RES[1]/8), 0, 8)

    <span class="pycomment"># setup the screen palette...</span>
    set_palette(screen_surface)
        
    <span class="pycomment"># apply the same palette to plasma_surface</span>
    plasma_surface.set_palette(screen_surface.get_palette())

    <span class="pycomment"># Precalculate the consine waves...</span>
    make_cosine()	

    <span class="pycomment"># Fruity loops...</span>
    while 1:
    
        <span class="pycomment"># Have we received an event to close the window?</span>
        for e in pygame.event.get():
            if e.type in (QUIT,KEYDOWN,MOUSEBUTTONDOWN):
                return
                
        <span class="pycomment"># Nope, lets sum the cosine values and update our plasma_array </span>
        add_cosine(plasma_buffer)
        <span class="pycomment"># Now, blit the arrary to the surface, scale it...</span>
        blit_scaled_surface(screen_surface, plasma_buffer, plasma_surface)
        <span class="pycomment"># Show the results to our audience...</span>
        pygame.display.flip()

            
<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def add_cosine(plasma_buffer):</span>
    <span class="pycomment">"An Y by X loop of screen co-ords, summing the values of four cosine values to produce a colour value that'll map to the previously set surface palette."</span>
	
    <span class="pycomment"># Use working indices for the cosine table, save the real ones for later...</span>
    t1 = pnt_tab[0]
    t2 = pnt_tab[1]
    <span class="pycomment"># Loop for all Y screen coords...</span>
    for y in range(0,RES[1]/8):

        <span class="pycomment"># Save the horizontal indices for later use...</span>
        t3 = pnt_tab[2]
        t4 = pnt_tab[3]
        <span class="pycomment"># Loop accross the screen...</span>
        for x in range(0,RES[0]/8):
            <span class="pycomment"># Our colour value will equal the sum of four cos_table offsets. </span>
            <span class="pycomment"># The preset surface palette comes in handy here! We just need to output the value...</span>
            <span class="pycomment"># We mod by 256 to prevent our index going out of range. (C would rely on 8bit byte ints and with no mod?)</span>
            colour = cos_tab[mod(t1,256)] + cos_tab[mod(t2,256)] + cos_tab[mod(t3,256)] + cos_tab[mod(t4,256)]
                    
            <span class="pycomment"># Arbitrary values, changing these will allow for zooming etc...</span>
            t3 += 3
            t4 += 2
                        
            <span class="pycomment"># Insert the calculated colour value into our working surfarray...</span>
            plasma_buffer[x][y] = colour

        <span class="pycomment"># Arbitrary values again...</span>
        t1 += 2
        t2 += 1
        
    <span class="pycomment"># Arbitrary values to move along the cos_tab. Play around for something nice...</span>
    <span class="pycomment"># Don't think I need these boundary checkings, but just in case someone decides to run this code for a couple of weeks non-stop...</span>
    <span class="pycomment">#</span>
    if(pnt_tab[0] < 256): 
        pnt_tab[0] += 1
    else:
        pnt_tab[0] = 1
        
    if(pnt_tab[1] < 256):
        pnt_tab[1] += 2
    else:
        pnt_tab[1] = 2
        
    if(pnt_tab[2] < 256):
        pnt_tab[2] += 3
    else:
        pnt_tab[2] = 3
        
    if(pnt_tab[3] < 256):
        pnt_tab[3] += 4
    else:
        pnt_tab[3] = 4
        	
<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def make_cosine():</span>
    <span class="pycomment">"Knock up a little pre-calculated cosine lookup table..."</span>
    i = 0
    for i in range (0,256):
        <span class="pycomment"># Play with the values here for interesting results... I just made them up! :-)</span>
        cos_tab[i]=60*(cos(i*PI/32))


<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def set_palette(screen_surface):</span>
    <span class="pycomment">"Create something trippy... Based on Pete's cmap creator, and without doubt the thing that took the longest... Aaaargh! Decent palettes are hard to find..."</span>
    colour_map = zeros((256, 3))
    
    i=0
    <span class="pycomment"># We're trying to compress as large a range of colours into an 8bit palette as possible...</span>
    <span class="pycomment"># so we go for a typical RGB spread. </span>
    <span class="pycomment"># A larger 2 x 2 colour range over 128 indices also works well...</span>
    <span class="pycomment"># Have a play and see what you like! :-)</span>
    for i in range(0,64):
        colour_map[i][0] = 255
        colour_map[i][1] = i * 4
        colour_map[i][2] = 255 - (i * 4)
        
        colour_map[i+64][0] = 255 - (i * 4)     
        colour_map[i+64][1] = 255 
        colour_map[i+64][2] = (i * 4)
            
        colour_map[i+128][0] = 0     
        colour_map[i+128][1] = 255 - (i * 4)
        colour_map[i+128][2] = 255
     
        colour_map[i+192][0] = i * 4        
        colour_map[i+192][1] = 0  
        colour_map[i+192][2] = 255    
        
            
    <span class="pycomment"># Apply our palette to the screen's surface...</span>
    screen_surface.set_palette(colour_map)
    print colour_map
    
    
    
    
<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def blit_scaled_surface(screen, flame, miniflame):</span>
    <span class="pycomment">"double the size of the data, and blit to screen -- Nicked from Shread's Fast Flame"</span>
    blit_array(miniflame, flame)
    s2 = pygame.transform.scale(miniflame, screen.get_size())
    screen.blit(s2, (0,0))

<span class="pycomment"># Shit captain! In the void of space we're all alone...</span>
<span class="pycomment"># Don't worry ensign. I have a plan: Fire our trippy laser!</span>
if __name__ == '<span class="pystring">__main__</span>': main()

<span class="pycomment"># End of sauce. Pass the chips...</span>

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

