<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  pygame Code Profiling class</H3>
            <p>Author: Kevin Turner (acapnotic at users.sourceforge.net)<br>
            Submission date: May 29, 2001</p>
	  
            <p>Description: profile.Profile sub-class using pygame.time.get_ticks.</p>

            <p>Download: <a href="prof.py">prof.py</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: Any</p>

            <p>Comments: Comments: The standard profile module uses the python timer; Kevin's code replaces it with the pygame.time timer. Note that, as with the standard profiler, machine-specific calibration may result in more accurate measurements - check the profile.py module <a href="http://www.python.org/doc/current/lib/profile-calibration.html">documentation</a> for more information.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment"># profile.Profile sub-class using pygame.time.get_ticks.
# by Kevin.  Public domain.  Have fun.</span>

import profile, pstats
import pygame

<span class="pydef">def f8_i(x):</span>
    <span class="pycomment">"""using an integer clock, so lose the floating-point formatting."""</span>

    return string.rjust(`<span class="pystring">x</span>`,8)

pstats.f8 = f8_i

<span class="pydef">class PygameProfile(profile.Profile):</span>
    <span class="pycomment">"""Profile sub-class using pygame.time.get_ticks.

    Seems to be pretty fast and precise.
    """</span>

    <span class="pydef">def __init__(self):</span>
        profile.Profile.__init__(self, pygame.time.get_ticks)

    <span class="pydef">def trace_dispatch_i(self, frame, event, arg):</span>
        t = self.timer() - self.t <span class="pycomment"># - 0 # Calibration constant</span>
        <span class="pycomment"># (but this needs no calibration constant on my machine)</span>

        if self.dispatch[event](frame,t):
            self.t = self.timer()
        else:
            self.t = self.timer() - t <span class="pycomment"># put back unrecorded delta</span>
        return


<span class="pydef">def example():</span>
    pr=PygameProfile()
    pr.run('<span class="pystring">function_to_profile(with, "args", as_needed)</span>')
    pr.dump_stats('<span class="pystring">my_function.prof</span>')

    <span class="pycomment">    # To display:

    # Also could be pstats.Stats(pr), if you still have the
    # Profile object around at this point.</span>
    s = pstats.Stats('<span class="pystring">my_function.prof</span>')

    s.strip_dirs().sort_stats('<span class="pystring">time</span>')
    s.print_stats()

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

