<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  'Selfrep' Effect Script</H3>
            <p>Author: Sebastian &quot;WhiteTiger&quot; John (TheWhiteTiger at gmx.net)<br>
            Submission date: September 27, 2002</p>
	  
            <p>Description: Three different experiments with self-representation.</p>
            <p align="center"><img src="selfrep.png" width="300" height="200"></p>

            <p>Download: <a href="selfrep.zip">selfrep.zip</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: Any</p>

            <p>Comments: Self-representation is produced when an effect's output is turned into its input - under these conditions, recursion results, and small changes can have dramatic results. Mr. John demonstrates this with the following script, which has three 'flavors' - use the 1, 2, and 3 keys to cycle through them.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""
Name:		Selfrep
Author:		Sebastian "WhiteTiger" John
Email:		TheWhiteTiger@gmx.net

This is public domain code.

This effect (my first one!!! ;) does nothing more than take a surface (in
this example a rectangle), rotate it, scale it down, and put the rectangle
around it again, and then draw it... The three different transform functions
in this file should do almost the same, but they look quite a bit different.

I got the idea for this effect from 'Gdel, Escher, Bach' by Douglas R.
Hofstadter, in one chapter he wrote about self-replication, with the example
of a video camera directly connected to a TV, and then have the camera
capture the TV itself... I wanted to get this with my computer, too, but it
seems to be a lot harder than I thought... This effect is not what I really
wanted, but it's nice, too... :)

Keyboard controls:
  "1" => transform effect one
  "2" => transform effect two
  "3" => transform effect three
  "q" => quit (also working: mouse click, Escape)
"""</span>

import pygame
from pygame.locals import *

<span class="pycomment"># screen resolution</span>
RES = RESX, RESY = (300, 200)
SCR_RECT = (0, 0, RESX, RESY)

<span class="pycomment"># configuration flags</span>
INITIAL_RECT = 1
RESET_ANGLE = 0

<span class="pycomment"># drawing color... you could also change it to 'COLOR = (255,0,0)'...</span>
COLOR = (255, 255, 255)

<span class="pycomment"># rotozoom parameters</span>
ANGLE = 0
SCALE = 0.94
<span class="pycomment"># or .8, .9, .92, .95, ...</span>

<span class="pydef">def inc_angle(step=1):</span>
    global ANGLE
    ANGLE += step
    if ANGLE > 360: ANGLE -= 360

<span class="pydef">def transform1(surf):</span>
    <span class="pycomment"># 0.92 looks quite better than the default of 0.94...</span>
    surf = pygame.transform.rotozoom(surf, 5, 0.92)
    pygame.draw.rect(surf, COLOR, SCR_RECT, 10)
    return surf

<span class="pydef">def transform2(surf):</span>
    inc_angle()
    surf = pygame.transform.rotozoom(surf, ANGLE, SCALE)
    newsurf = pygame.Surface(RES)
    x, y = (RESX-RESX*SCALE)/2, (RESY-RESY*SCALE)/2
    newsurf.blit(surf, (x, y))
    pygame.draw.rect(newsurf, COLOR, SCR_RECT, 10)
    return newsurf

<span class="pydef">def transform3(surf):</span>
    inc_angle()
    surf = pygame.transform.rotozoom(surf, ANGLE, SCALE)
    surf = pygame.transform.scale(surf, RES)
    pygame.draw.rect(surf, COLOR, SCR_RECT, 10)
    return surf

<span class="pydef">def init_surface():</span>
    surf = pygame.Surface(RES)
    if INITIAL_RECT:
        pygame.draw.rect(surf, COLOR, SCR_RECT, 10)
    return surf

<span class="pydef">def new_transform(key):</span>
    if key == K_1:
        transform = transform1
    elif key == K_2:
        transform = transform2
    elif key == K_3:
        transform = transform3
    
    if RESET_ANGLE:
        global ANGLE
        ANGLE = 0
    
    surf = init_surface()
    
    return transform, surf

<span class="pydef">def main():</span>
    <span class="pycomment"># initialize the screen</span>
    pygame.init()
    screen = pygame.display.set_mode(RES)
    
    <span class="pycomment"># initial transform function and surface</span>
    transform_surface = transform3
    surf = init_surface()
    
    done = 0
    while not done:
        <span class="pycomment"># keep an eye on the events</span>
        for ev in pygame.event.get():
            if ev.type in (QUIT, MOUSEBUTTONDOWN):
                done = 1
            elif ev.type == KEYDOWN:
                if ev.key in (K_ESCAPE, K_q):
                    done = 1
                elif ev.key in (K_1, K_2, K_3):
                    transform_surface, surf = new_transform(ev.key)
        
        <span class="pycomment"># do all the nice stuff with the surface</span>
        surf = transform_surface(surf)
        
        <span class="pycomment"># draw it</span>
        screen.blit(surf, (0, 0))
        pygame.display.flip()
        pygame.time.wait(50)

if __name__ == "<span class="pystring">__main__":</span>
    main()

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

