<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  2d Vector Class</H3>
            <p>Author: Chris Wood (<a href="http://grace.2ya.com">http://grace.2ya.com</a>)<br>
            Submission date: September 20, 2002</p>
	  
            <p>Description: A simple 2d vector class</p>
            
            <p>Download: <a href="vector.py">vector.py</a></p>	  

            <p>pygame version required: None<br>
            SDL version required: None<br>
            Python version required: 2.0</p>

            <p>Comments: Here's a small class that provides the basics of 2d vector math. Adding other functions like vector multiplication and dot product calculation should be simple if you follow the outline of this class.</p>
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""
vector.py
A simple physics 2D vector class
Licence: Public Domain
Author: Chris Wood (http://grace.2ya.com)
    20 September 2002

Any game involving 2D physics could do with a class similar to this one. This
class is as fast as possible while still providing all the functionality you
would commonly need.

Note that storing the vector as cartesian coordinates (rather than polar or
both) makes for faster for vector addition. It is assumed that you won't often
need the polar coordinates.
"""</span>


from math import sin, cos, atan, hypot

<span class="pycomment"># The quick way to know how big the pie is</span>
PI = 3.141592653589793238462643383
TwoPI = PI * 2.0
HalfPI = PI * 0.5
OneAndHalfPI = PI * 1.5

<span class="pycomment"># new math function</span>
<span class="pydef">def direction(x, y):</span>
    <span class="pycomment">"""Return the direction component of a vector (in radians), given
    cartesian coordinates.
    """</span>
    if x > 0:
        if y >= 0:
            return atan(y / x)
        else:
            return atan(y / x) + TwoPI
    elif x == 0:
        if y > 0:
            return HalfPI
        elif y == 0:
            return 0
        else:
            return OneAndHalfPI
    else:
        return atan(y / x) + PI


<span class="pydef">class Vector:</span>
    <span class="pycomment">"""Store a vector in cartesian coordinates."""</span>
    
    <span class="pydef">def __init__(self, f = 0.0, mag = 0.0):</span>
        <span class="pycomment">"""Create a Vector, given polar coordinates."""</span>
        <span class="pycomment"># Calculate cartesian coordinates</span>
        self.x = mag * cos(f)
        self.y = mag * sin(f)

    <span class="pydef">def add(self, b):</span>
        <span class="pycomment">"""Add b to self, where b is another Vector."""</span>
        self.x += b.x
        self.y += b.y
        
    <span class="pydef">def heading(self):</span>
        <span class="pycomment">"""Return the direction of the Vector in radians."""</span>
        return direction(self.x, self.y)
        
    <span class="pydef">def mag(self):</span>
        <span class="pycomment">"""Return the magnitude of the Vector."""</span>
        return hypot(self.x, self.y)

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

