<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title: Zipshow </H3>
            <p>Author: Pete Shinners (pete at shinners.org)<br>
            Submission date: May 21, 2002</p>
	  
            <p>Description: Displays any images contained in a .zip archive.</p>

            <p>Download: <a href="zipshow.py">zipshow.py</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: Any</p>

            <p>Comments: Many game projects require a very large number of graphics files - sprite images, backgrounds, textures and other such resources.  Some developers simply ship their games with the graphics files as-is, while others prefer to group them together in compressed archives. While it won't keep your distribution size down (unless you're using an uncompressed format like .bmp), python's zipfile module makes it easy to ship your graphics resources in one easy-to handle, familiar archive format.</p>

	  <p>The zipshow script will display each stored file in a zip archive, one at a time - hit a key or click the mouse to page through them, or hit ESC to quit.  Place it in a directory with an archive, or feed it the name of a zip file, and watch it do its thing.</p>
      
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">
#/usr/bin/env python

"""
Pete Shinners
May 20, 2002

This shows how to load images from a zipfile. The script itself
will take the name of a zip file and display all the image inside
the zip.

You will want to change to code yourself to suit your game,
but this script shows you how to find images inside a zip file
and then load a specific image from the zip.

This example originally was included as an example with earlier
versions of pygame.
"""</span>



import zipfile, sys, glob, pygame
from cStringIO import StringIO
from pygame.locals import *



<span class="pydef">def showimage(imgsurface, name):</span>
    <span class="pycomment">"show a loaded image onto the screen"</span>
    size = imgsurface.get_size()
    pygame.display.set_caption(name+'  '+`<span class="pystring">size</span>`)
    screen = pygame.display.set_mode(size)
    screen.blit(imgsurface, (0,0))
    pygame.display.flip()

    <span class="pycomment">#loop through events until done with this pic</span>
    while 1:
        e = pygame.event.wait()
        if e.type == QUIT:
            return 0
        if e.type in (KEYDOWN, MOUSEBUTTONDOWN):
            break
    return 1


<span class="pydef">def zipslideshow(zipfilename):</span>
    <span class="pycomment">"loop through all the images in the zipfile"</span>
    zip = zipfile.ZipFile(zipfilename)
    for file in zip.namelist():
        <span class="pycomment">#get data from zipfile</span>
        data = zip.read(file)

        <span class="pycomment">#create stringio for data (a file-like object)</span>
        data_io = StringIO(data)

        <span class="pycomment">#load from a stringio object (ignore error exceptions)</span>
        try: surf = pygame.image.load(data_io, file)
        except: continue

        <span class="pycomment">#show image on the screen</span>
        if not showimage(surf, file):
            return


<span class="pydef">def main():</span>
    <span class="pycomment">"run the program, handle arguments"</span>
    pygame.init()
    zipfiles = sys.argv[1:]
    if not zipfiles:
        zipfiles = glob.glob('<span class="pystring">*.zip</span>') <span class="pycomment">#find any zipfile in this directory</span>
    if not zipfiles:
        raise SystemExit('<span class="pystring">No ZIP files given, or in current directory</span>')
    zipslideshow(zipfiles[0])


<span class="pycomment">#run the script if not being imported</span>
if __name__ == '<span class="pystring">__main__</span>': main()

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

